 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#ifndef __LIN_H__
#define __LIN_H__

#include "mf_config.h"

/* 功能支持 */
#define AUTOBAUD            1   /* 0-固定波特率 1-自适应波特率 */

/* LIN 相关宏定义 */
#define MAIN_CLOCK          __SYSTEM_CLOCK                  /* 主频, 单位Hz */
#define TIMER_1MS_LOAD      (MAIN_CLOCK/1000)               /* 定时器1ms, Load值 */ 
#define TIMER_1US_LOAD      (MAIN_CLOCK/1000000)            /* 定时器1us, Load值 */
#define TIME_BASE_PERIOD    500                             /* 接收超时定时器, 单位us */
#define TIMER_LOAD_VALUE    TIMER_1US_LOAD * TIME_BASE_PERIOD
#define LIN_IDLE_TIMEOUT    10000                           /* TIME_BASE_PERIOD为单位,休眠周期,此时间段内无通信则进入sleep,有通信则重置超时 */

/* 校验类型 */
#define CHECKSUM_TYPE_STANDARD   0
#define CHECKSUM_TYPE_ENHANCED   1
#define CHECKSUM_TYPE            CHECKSUM_TYPE_STANDARD     /* 0-标准型 1-增强型 */

/* Master 定义ID */
typedef enum {
    LIN_MASTER_CMD0 = 0x17,
    LIN_MASTER_CMD1 = 0x37,  
    LIN_MASTER_CMD2 = 0x38,  
    /* 此处增加自定义ID */
}lin_master_cmd_t;

/* Slave 定义ID */
typedef enum {
    LIN_SLAVE_CTRL_REQUEST = 0x17,
    LIN_SLAVE_CTRL_RESPONSE = 0x37,  
    LIN_SLAVE_CMD2 = 0x38,
	LIN_SLAVE_BOOT_REQUEST = 0x3C,
	LIN_SLAVE_BOOT_RESPONSE = 0x3D  
    /* 此处增加自定义ID */
}lin_slave_cmd_t;

typedef enum {
    TRANSMIT,   /* 主机发送 */
    RECEIVE     /* 主机接收 */
}lin_packet_type_t;

/* 主机帧结构 */
typedef struct {
    uint8_t cmd;           /* 参考LIN_MASTER_CMD0 */
    lin_packet_type_t type;/* 命令类型 */
    uint8_t length;
    uint8_t timeout;       /* 超时/ms, 可在帧时隙到达之前提前判定任务接收 */
    uint8_t period;        /* 周期/ms */
    uint8_t* data;
    uint8_t pid;           /* 转换cmd为pid, 初始化时进行 */
}lin_master_cmd_packet_t;

/* 从机帧结构 */
typedef struct {
    uint8_t cmd;           /* 参考LIN_SLAVE_CTRL_REQUEST */
    lin_packet_type_t type;/* 命令类型 */
    uint8_t length;
    uint8_t* data;
    uint8_t pid;           /* 转换cmd为pid, 初始化时进行 */
	volatile uint8_t FullFlg;   /* 有数据标记 */
}lin_slave_cmd_packet_t;

#define LIN_BREAK_BIT       13                                          /* 同步间隔, 13bit显性电平 */
#define LIN_BREAK_BYTE      0x00
#define LIN_SYN_BYTE        0x55

/* lin波特率和字节间间隔 */
#define LIN_BAUDRATE                    19200                           /* 波特率, 上限20K */
#define LIN_RECV_BYTE_TIME_SPACE        12                              /* 接收字节间间隔, 最大支持255bit */

#define LIN_SYN_BYTE_TIME_SPACE         50                              /* 收到间隔段之后 */
#define LIN_BIT_TIMER_LOAD  (MAIN_CLOCK / LIN_BAUDRATE)                 /* 比特定时器Load值, TIME时钟源24MHz */
#define RESPONSE_SPACE      (50 * LIN_BIT_TIMER_LOAD)                   /* 应答间隔, 5字节长度, 每字节10bit */

//Slave UART配置
#define LIN_SLAVE_UART                          UART0
#define LIN_SLAVE_UART_BASE                     UART0_BASE
#define LIN_SLAVE_UART_RX_GPIO                  GPIOA
#define LIN_SLAVE_UART_RX_PIN                   FL_GPIO_PIN_2
#define LIN_SLAVE_UART_RX_REMAP_FUNCTION        FL_GPIO_PINREMAP_FUNCTON0
#define LIN_SLAVE_UART_TX_GPIO                  GPIOA
#define LIN_SLAVE_UART_TX_PIN                   FL_GPIO_PIN_3
#define LIN_SLAVE_UART_TX_REMAP_FUNCTION        FL_GPIO_PINREMAP_FUNCTON0
#define LIN_SLAVE_UART_RX_PIN_FCR_MASK          0x00000030                      /* (3 << (LIN_SLAVE_UART_RX_PIN_IDX << 1)) */
#define LIN_SLAVE_UART_RX_PIN_FCR_MASK_INV      0xFFFFFFCF                      /* 取反 */
#define LIN_SLAVE_UART_RX_PIN_FCR_INPUT         0                               /* (0 << (LIN_SLAVE_UART_RX_PIN_IDX << 1)) */
#define LIN_SLAVE_UART_RX_PIN_FCR_DIGIT         0x00000020                      /* (2 << (LIN_SLAVE_UART_RX_PIN_IDX << 1)) */
#define LIN_SLAVE_RX_EXTI_INPUT_GROUP           FL_GPIO_EXTI_INPUT_GROUP0
#define LIN_SLAVE_RX_EXTI_LINE                  FL_GPIO_PIN_2
#define LIN_SLAVE_RX_EXTI_LINE_PIN              FL_GPIO_PIN_2
#define LIN_SLAVE_RX_EXTI_LINE_INDEX            2
#define LIN_SLAVE_RX_EXTI_EDS_INDEX             4
#define LIN_SLAVE_UART_IRQHandler               UART0_IRQHandler
#define LIN_SLAVE_UART_IRQType                  UART0_IRQn

#define LIN_SLAVE_SLP_GPIO                      GPIOF
#define LIN_SLAVE_SLP_PIN                       FL_GPIO_PIN_8
#define LIN_SLAVE_SLP_ENABLE()                  FL_GPIO_ResetOutputPin(LIN_SLAVE_SLP_GPIO, LIN_SLAVE_SLP_PIN)
#define LIN_SLAVE_SLP_DISABLE()                 FL_GPIO_SetOutputPin(LIN_SLAVE_SLP_GPIO, LIN_SLAVE_SLP_PIN)

//Master UART配置
#define LIN_MASTER_UART                        UART1
#define LIN_MASTER_UART_BASE                   UART1_BASE
#define LIN_MASTER_UART_RX_GPIO                GPIOB
#define LIN_MASTER_UART_RX_PIN                 FL_GPIO_PIN_13
#define LIN_MASTER_UART_RX_REMAP_FUNCTION      FL_GPIO_PINREMAP_FUNCTON0
#define LIN_MASTER_UART_TX_GPIO                GPIOB
#define LIN_MASTER_UART_TX_PIN                 FL_GPIO_PIN_14
#define LIN_MASTER_UART_TX_REMAP_FUNCTION      FL_GPIO_PINREMAP_FUNCTON0
#define LIN_MASTER_UART_RX_PIN_FCR_MASK        0x0C000000                      /* (3 << (LIN_MASTER_UART_RX_PIN_IDX << 1)) */
#define LIN_MASTER_UART_RX_PIN_FCR_MASK_INV    0xF3FFFFFF                      /* 取反 */
#define LIN_MASTER_UART_RX_PIN_FCR_INPUT       0                               /* (0 << (LIN_MASTER_UART_RX_PIN_IDX << 1)) */
#define LIN_MASTER_UART_RX_PIN_FCR_DIGIT       0x08000000                      /* (2 << (LIN_MASTER_UART_RX_PIN_IDX << 1)) */
#define LIN_MASTER_RX_EXTI_INPUT_GROUP         FL_GPIO_EXTI_INPUT_GROUP0
#define LIN_MASTER_RX_EXTI_LINE                FL_GPIO_EXTI_PIN_13
#define LIN_MASTER_RX_EXTI_LINE_PIN            FL_GPIO_EXTI_PIN_13
#define LIN_MASTER_RX_EXTI_LINE_INDEX          13
#define LIN_MASTER_RX_EXTI_EDS_INDEX           26
#define LIN_MASTER_UART_IRQHandler             UART1_IRQHandler
#define LIN_MASTER_UART_IRQType                UART1_IRQn

#define LIN_MASTER_SLP_GPIO                    GPIOF
#define LIN_MASTER_SLP_PIN                     FL_GPIO_PIN_8
#define LIN_MASTER_SLP_ENABLE()                FL_GPIO_ResetOutputPin(LIN_MASTER_SLP_GPIO, LIN_MASTER_SLP_PIN)
#define LIN_MASTER_SLP_DISABLE()               FL_GPIO_SetOutputPin(LIN_MASTER_SLP_GPIO, LIN_MASTER_SLP_PIN)

/* 用户处理 */
/* Slave */
#define LIN_SLAVE_CTRL_REQUEST_LENGTH  8
#define LIN_SLAVE_CTRL_RESPONSE_LENGTH  8
#define LIN_SLAVE_CMD2_LENGTH  8
#define LIN_SLAVE_BOOT_REQUEST_LENGTH  8
#define LIN_SLAVE_BOOT_RESPONSE_LENGTH  8

extern uint8_t LIN_SLAVE_CTRL_REQUEST_Data[8];
extern uint8_t LIN_SLAVE_CTRL_RESPONSE_Data[8];
extern uint8_t LIN_SLAVE_CMD2_Data[8];
extern uint8_t LIN_SLAVE_BOOT_REQUEST_Data[8];
extern uint8_t LIN_SLAVE_BOOT_RESPONSE_Data[8];

extern lin_slave_cmd_packet_t slave_scheduleTable[];
#define SLAVE_TABLE_SIZE  (sizeof(slave_scheduleTable)/sizeof(lin_slave_cmd_packet_t))
    
/* Master */
#define LIN_MASTER_CMD0_LENGTH     8
#define LIN_MASTER_CMD0_TIMEOUT    20
#define LIN_MASTER_CMD0_PERIOD     100 /* 最大255ms */

#define LIN_MASTER_CMD1_LENGTH     8
#define LIN_MASTER_CMD1_TIMEOUT    30
#define LIN_MASTER_CMD1_PERIOD     200 /* 最大255ms */

#define LIN_MASTER_CMD2_LENGTH     8
#define LIN_MASTER_CMD2_TIMEOUT    40
#define LIN_MASTER_CMD2_PERIOD     200 /* 最大255ms */

extern uint8_t LIN_MASTER_CMD0_Data[8];
extern uint8_t LIN_MASTER_CMD1_Data[8];
extern uint8_t LIN_MASTER_CMD2_Data[8];

extern lin_master_cmd_packet_t master_scheduleTable[];
#define MASTER_TABLE_SIZE  (sizeof(master_scheduleTable)/sizeof(lin_master_cmd_packet_t))

void lin_init(void);

#if (SLEEP_SUPPORT == 1)
void lin_master_sleep_init(void);
void lin_master_awake_init(uint8_t awake_type);
uint8_t lin_master_is_sleep(void);
void lin_slave_sleep_init(void);
void lin_slave_awake_init(uint8_t awake_type);
uint8_t lin_slave_is_sleep(void);
#endif

extern int32_t LIN_SLAVE_BOOT_TxMsgFill(uint8_t *data, uint32_t lenth, uint32_t dummy);

#endif
