 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "adc.h"
#include "mf_config.h"

#define ADC_DMA_LENGTH 20
#define ADC_EXTERNAL_GROUP      FL_ADC_EXTERNAL_CH0|FL_ADC_EXTERNAL_CH1|FL_ADC_EXTERNAL_CH5|FL_ADC_EXTERNAL_CH6|FL_ADC_EXTERNAL_CH7| \
                                FL_ADC_EXTERNAL_CH8|FL_ADC_EXTERNAL_CH11|FL_ADC_EXTERNAL_CH12|FL_ADC_EXTERNAL_CH13|FL_ADC_EXTERNAL_CH15
                    
volatile uint16_t  ADC_Result[2][ADC_DMA_LENGTH/2] ={0};

#define FIRST_CH_SELECT		1
#define FIRST_CH_START		2
#define FIRST_CH_END		3
#define SECOND_CH_SELECT	4
#define SECOND_CH_START		5
#define CAPTURE_END			6

uint8_t uState = 0;				//采集状态机

uint16_t STABLEDLY_SINGLE = 280;	//前续通道等待稳定延时
uint16_t STABLEDLY_SECOND = 250;	//后续通道等待稳定延时

//采集需要的最大时间			4500/32000000 = 0.24ms		该时间设置必须大于实际采集时间
uint16_t SAMPLEDLY = 240;

//同时采集两通道时  通道切换延时
uint16_t CHANGEDLY = 32;

//步骤延时uS
#define StepDelay(x)	{FL_BSTIM16_WriteCounter(BSTIM16,(FL_BSTIM16_ReadAutoReload(BSTIM16)-x));}

void StateFun(void)
{
    switch(uState)
    {
        case FIRST_CH_SELECT:
            StepDelay(STABLEDLY_SINGLE);  //0.2ms
            uState = FIRST_CH_START;
            break;

        case FIRST_CH_START:
            ADC_Start(); //开始转换(触发ADC转换)
            StepDelay(SAMPLEDLY); //0.14ms(等待最大采样时间)
            uState = FIRST_CH_END;
            break;

        case FIRST_CH_END:
            StepDelay(CHANGEDLY);     //32us
            uState = SECOND_CH_SELECT;
            break;

        case SECOND_CH_SELECT:
            StepDelay(STABLEDLY_SECOND);
            uState = SECOND_CH_START;
            break;

        case SECOND_CH_START:
            ADC_Start(); //开始转换(触发ADC转换)
            StepDelay(SAMPLEDLY); //0.14ms(等待最大采样时间)
            uState = CAPTURE_END;
            break;

        case CAPTURE_END:
            FL_BSTIM16_Disable(BSTIM16);	        //关闭定时器
            FL_BSTIM16_DisableIT_Update(BSTIM16);   //禁止定时器中断
            break;

        default:
            FL_BSTIM16_Disable(BSTIM16);	        //关闭定时器
            FL_BSTIM16_DisableIT_Update(BSTIM16);   //禁止定时器中断
            break;
    }
}

//尽可能缩短工作时间可以降低功耗(采集结束提前)
static void ATDResFun(void)
{
    if((uState == FIRST_CH_END) || (uState == CAPTURE_END))		//采集完成且DMA中断完成则提前结束采样等待
    {
        StepDelay(0x10);
    }
}

void CapCaptureInit(void)
{
    FL_BSTIM16_InitTypeDef    TimerBase_InitStruct = {0};

    /*选择时钟源*/
    TimerBase_InitStruct.clockSource = FL_CMU_BSTIM16_CLK_SOURCE_APBCLK;
    /*配置预分频*/
    TimerBase_InitStruct.prescaler = __SYSTEM_CLOCK_1MHz_PRE - 1;    /* 分频后时钟为1M */
    /*配置预自动重载值*/
    TimerBase_InitStruct.autoReload = BSTIM16_AUTORELOAD_VALUE - 1;     /* 最长定时1000us = 1ms */
    /*使能自动重载*/
    TimerBase_InitStruct.autoReloadState = FL_ENABLE;

    FL_BSTIM16_Init(BSTIM16, &TimerBase_InitStruct);
    FL_BSTIM16_SetTriggerOutput(BSTIM16, FL_BSTIM16_TRGO_UPDATE);

    /* 清除计数器中断标志位 */
    FL_BSTIM16_ClearFlag_Update(BSTIM16);
    /* 开启计数器中断 */
    FL_BSTIM16_EnableIT_Update(BSTIM16);
  
    /* 中断优先级配置 */
    NVIC_DisableIRQ(LPTIM_BSTIM_IRQn);
    NVIC_SetPriority(LPTIM_BSTIM_IRQn, LPTIM_BSTIM_IRQ_PRI);
    NVIC_EnableIRQ(LPTIM_BSTIM_IRQn);

    /* 使能定时器 */
    FL_BSTIM16_Enable(BSTIM16);

}

void KickSensorper(void)
{
    uState = FIRST_CH_SELECT;

    StateFun();			//开启采集状态机

    FL_BSTIM16_EnableIT_Update(BSTIM16);    //中断使能
    FL_BSTIM16_Enable(BSTIM16);             //定时器启动
}

/**
  * @brief  BSTIM中断服务函数
  * @param  void
  * @retval void
  */

void BSTIM_IRQHandler(void)
{
    if(FL_BSTIM32_IsEnabledIT_Update(BSTIM32) && FL_BSTIM32_IsActiveFlag_Update(BSTIM32))
    {
        FL_BSTIM32_ClearFlag_Update(BSTIM32);
        KickSensorper();
    }

    if(FL_BSTIM16_IsActiveFlag_Update(BSTIM16))
    {
        FL_BSTIM16_ClearFlag_Update(BSTIM16);
        StateFun();
    }
}
/**
  * @brief  BSTIM32初始化
  * @param  void
  * @retval void
  */
void BSTIM32_Init(void)
{
    FL_BSTIM32_InitTypeDef    TimerBase_InitStruct;

    /*选择时钟源*/
    TimerBase_InitStruct.clockSource = FL_CMU_BSTIM32_CLK_SOURCE_APBCLK;
    /*配置预分频*/
    TimerBase_InitStruct.prescaler = __SYSTEM_CLOCK_1MHz_PRE - 1;
    /*配置预自动重载值*/
    TimerBase_InitStruct.autoReload = BSTIM32_AUTORELOAD_VALUE - 1;
    /*使能自动重载*/
    TimerBase_InitStruct.autoReloadState = FL_ENABLE;

    FL_BSTIM32_Init(BSTIM32, &TimerBase_InitStruct);

    /* 清除计数器中断标志位 */
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
    /* 开启计数器中断 */
    FL_BSTIM32_EnableIT_Update(BSTIM32);
  
    /* 中断优先级配置 */
    NVIC_DisableIRQ(LPTIM_BSTIM_IRQn);
    NVIC_SetPriority(LPTIM_BSTIM_IRQn, LPTIM_BSTIM_IRQ_PRI);
    NVIC_EnableIRQ(LPTIM_BSTIM_IRQn);

    /* 使能定时器 */
    FL_BSTIM32_Enable(BSTIM32);
}

/**
  * @brief  BSTIM32逆初始化
  * @param  void
  * @retval void
  */
void BSTIM32_DeInit(void)
{
    FL_BSTIM32_DeInit(BSTIM32);

    NVIC_ClearPendingIRQ(LPTIM_BSTIM_IRQn);

    NVIC_DisableIRQ(LPTIM_BSTIM_IRQn);
}

/**
  * @brief  ADC的DMA配置
  * @param  buffer: 数据指针
  *         length: 传输长度
  * @retval void
  */
void ADC_DMA(uint16_t *buffer, uint32_t length)
{
    FL_DMA_InitTypeDef DMA_InitStruct = {0};
    FL_DMA_ConfigTypeDef DMA_ConfigStruct = {0};
    DMA_InitStruct.periphAddress = FL_DMA_PERIPHERAL_FUNCTION1;
    DMA_InitStruct.direction = FL_DMA_DIR_PERIPHERAL_TO_RAM;
    DMA_InitStruct.memoryAddressIncMode = FL_DMA_MEMORY_INC_MODE_INCREASE;
    DMA_InitStruct.dataSize = FL_DMA_BANDWIDTH_16B;
    DMA_InitStruct.priority = FL_DMA_PRIORITY_HIGH;
    DMA_InitStruct.circMode = FL_ENABLE;
    FL_DMA_Init(DMA, &DMA_InitStruct, FL_DMA_CHANNEL_4);

    FL_DMA_Enable(DMA);

    DMA_ConfigStruct.memoryAddress = (uint32_t)buffer;
    DMA_ConfigStruct.transmissionCount = length - 1;
    FL_DMA_StartTransmission(DMA, &DMA_ConfigStruct, FL_DMA_CHANNEL_4);

    /*---------------- 中断配置 ----------------*/
    /* 清除标志 */
    FL_DMA_ClearFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4);     /* 清全程中断标志 */
    FL_DMA_ClearFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4); /* 清半程中断标志 */

    FL_DMA_EnableIT_TransferComplete(DMA,FL_DMA_CHANNEL_4);      /* 使能全程中断*/
    FL_DMA_EnableIT_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4);  /* 使能半程中断*/

    /* 使能并配置NVICNVIC */
    NVIC_DisableIRQ(DMA_IRQn);
    NVIC_SetPriority(DMA_IRQn, DMA_IRQ_PRI); /* 设置中断优先级 */
    NVIC_EnableIRQ(DMA_IRQn);
}

/**
  * @brief  ADC初始化
  * @param  void
  * @retval void
  */
void AdcInit(void)
{
    FL_GPIO_InitTypeDef         GPIO_InitStruct = {0};
    FL_ADC_CommonInitTypeDef    ADC_CommonInitStruct;
    FL_ADC_InitTypeDef          ADC_InitStruct ;

    /* 配置引脚为模拟功能 */
    /* ADC 引脚 PD9 PD10 PD14 PD15 PD0 PD1 PD2 */
    GPIO_InitStruct.pin        = FL_GPIO_PIN_9 | FL_GPIO_PIN_10 | FL_GPIO_PIN_14 | FL_GPIO_PIN_15 |
                                 FL_GPIO_PIN_0 | FL_GPIO_PIN_1  | FL_GPIO_PIN_2;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_GPIO_BOTH_DISABLE;
    GPIO_InitStruct.remapPin   = FL_GPIO_PINREMAP_FUNCTON0;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    FL_GPIO_Init(GPIOD, &GPIO_InitStruct);

    /* 引脚 PF6 PF7 */
    GPIO_InitStruct.pin        = FL_GPIO_PIN_6 | FL_GPIO_PIN_7;
    FL_GPIO_Init(GPIOF, &GPIO_InitStruct);

    /* ADC 时钟配置 */
    ADC_CommonInitStruct.clockSource            = FL_CMU_ADC_CLK_SOURCE_PLL;    /* ADC_CLK 时钟源 */
    ADC_CommonInitStruct.clockPrescaler         = FL_CMU_ADC_PSC_DIV2;          /* ADC_CLK 分频系数 */
    ADC_CommonInitStruct.referenceSource        = FL_ADC_REF_SOURCE_VDDA;       /* ADC正基准 */
    ADC_CommonInitStruct.negativeReferenceSource = FL_ADC_REF_NEGA_VSSA;        /* ADC负基准 */
    FL_ADC_CommonInit(&ADC_CommonInitStruct);

    /* ADC 寄存器设置 */
    ADC_InitStruct.conversionMode                   = FL_ADC_CONV_MODE_SINGLE;     /* 单次模式 */
    ADC_InitStruct.autoMode                         = FL_ADC_SINGLE_CONV_MODE_AUTO;/* 自动 */
    ADC_InitStruct.waitMode                         = FL_ENABLE;                   /* 等待 */
    ADC_InitStruct.overrunMode                      = FL_ENABLE;                   /* 覆盖上次数据 */
    ADC_InitStruct.scanDirection                    = FL_ADC_SEQ_SCAN_DIR_FORWARD; /* 通道正序扫描 */
    ADC_InitStruct.externalTrigConv                 = FL_ADC_RGTRIGGER_EDGE_NONE;  /* 禁止触发信号 */
    ADC_InitStruct.triggerSource                    = FL_ADC_TRGI_LUT0;            /* 触发信号选择 */
    ADC_InitStruct.fastChannelTime                  = FL_ADC_FAST_CH_SAMPLING_TIME_192_ADCCLK;/* 快速通道采样时间 */
    ADC_InitStruct.lowChannelTime                   = FL_ADC_SLOW_CH_SAMPLING_TIME_192_ADCCLK;/* 慢速通道采样时间 */
    ADC_InitStruct.oversamplingMode                 = FL_DISABLE;                   /* 过采样打开 关闭可缩短采样时间 */
    ADC_InitStruct.overSampingMultiplier            = FL_ADC_OVERSAMPLING_MUL_8X;  /* 8倍过采样 */
    ADC_InitStruct.oversamplingShift                = FL_ADC_OVERSAMPLING_SHIFT_3B;/* 数据右移 */
    
    FL_ADC_Init(ADC, &ADC_InitStruct);
    
    FL_ADC_EnableDMAReq(ADC);

    FL_PMU_EnableVPTATBuffer(PMU);                      /* 使能VPTAT Buffer */
    FL_PMU_EnableAVREFBuffer(PMU);                      /* 使能AVREF BUFFER  */

    FL_ADC_DisableExternalSequencerChannel(ADC, ADC_EXTERNAL_GROUP);      /* 外部通道选择 */
    FL_ADC_EnableExternalSequencerChannel(ADC, ADC_EXTERNAL_GROUP);   

    ADC_DMA((uint16_t *)(ADC_Result), ADC_DMA_LENGTH);

    FL_ADC_Enable(ADC);

    /*---------------- 中断配置 ----------------*/
    /* 清除标志 */
    FL_ADC_ClearFlag_EndOfConversion(ADC);                      /* 清标志 */

    /* 中断使能 */
//   FL_ADC_EnableIT_EndOfSequence(ADC);                  /* 使能序列完成中断*/
//   FL_ADC_EnableIT_Overrun(ADC);
//   FL_ADC_EnableIT_EndOfConversion(ADC);

    /* 使能并配置NVICNVIC */
    NVIC_DisableIRQ(ADC_IRQn);
    NVIC_SetPriority(ADC_IRQn, ADC_IRQ_PRI); /* 设置中断优先级 */
    NVIC_EnableIRQ(ADC_IRQn);
}


void ADC_Start(void)
{
    FL_ADC_DisableExternalSequencerChannel(ADC, ADC_EXTERNAL_GROUP);      /* 外部通道选择 */
    FL_ADC_EnableExternalSequencerChannel(ADC, ADC_EXTERNAL_GROUP);

    FL_ADC_ClearFlag_EndOfConversion(ADC);                      /* 清标志 */
    FL_ADC_Enable(ADC);                                         /* 启动ADC */
    FL_ADC_EnableRegularGroupSWConversion(ADC);                 /* 开始转换 */
}

/* DMA中断服务函数 */
void DMA_IRQHandler(void)
{
    if(FL_DMA_IsActiveFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4))
    {
//        FL_ADC_Disable(ADC);
//        FL_ADC_ClearFlag_EndOfConversion(ADC);
        FL_DMA_ClearFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4);     /* 清全程中断标志 */

        ATDResFun();
    }

    if(FL_DMA_IsActiveFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4))
    {
        //FL_ADC_ClearFlag_EndOfConversion(ADC);
        FL_DMA_ClearFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4); /* 清半程中断标志 */

        ATDResFun();
    }

    if(FL_DMA_IsActiveFlag_AddressError(DMA))
    {
        FL_DMA_ClearFlag_AddressError(DMA);    /* 清地址错误标志 */
    }
}

void ADC_IRQHandler(void)
{
//    if(FL_ADC_IsActiveFlag_EndOfConversion(ADC))
//    {
//        FL_ADC_ClearFlag_EndOfConversion(ADC);
//    }
//    if(FL_ADC_IsActiveFlag_Overrun(ADC))
//    {
//        FL_ADC_ClearFlag_Overrun(ADC);
//    }

    if(FL_ADC_IsActiveFlag_EndOfSequence(ADC))
    {
        FL_ADC_ClearFlag_EndOfSequence(ADC);
    }
}


/**
  * @brief  ADC复位函数
  * @param  void
  * @retval void
  */
void ADC_DeInit(void)
{
    /* ADC外设寄存器复位 */
    FL_ADC_DeInit(ADC);

    /* ADC时钟复位 */
    FL_ADC_CommonDeInit();

    /* 清除中断挂起 */
    NVIC_ClearPendingIRQ(ADC_IRQn);

    /* 关闭中断 */
    NVIC_DisableIRQ(ADC_IRQn);
}

#if (SLEEP_SUPPORT == 1)
/**
  * @brief  ADC休眠前初始化
  * @param  void
  * @retval void
  */
void adc_sleep_init(void)
{
    /* 关闭Buffer功能 */
    FL_PMU_DisableVPTATBuffer(PMU);                      /* 禁能VPTAT Buffer */
    FL_PMU_DisableAVREFBuffer(PMU);                      /* 禁能AVREF BUFFER  */
    
    /* 关闭非唤醒外设 */
    FL_BSTIM32_Disable(BSTIM32);  /* 禁能定时器 *///TODO 需要验证关闭外设时机
    FL_BSTIM32_DisableIT_Update(BSTIM32);
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
//    Close_And_ClearIRQ(LPTIM_BSTIM_IRQn);
    NVIC_ClearPendingIRQ(LPTIM_BSTIM_IRQn);/* 须保留LPTIM16的NVIC使能 */
    
    FL_BSTIM16_Disable(BSTIM16);  /* 禁能定时器 */
    FL_BSTIM16_DisableIT_Update(BSTIM16);
    FL_BSTIM16_ClearFlag_Update(BSTIM16);
//    Close_And_ClearIRQ(LPTIM_BSTIM_IRQn);
    NVIC_ClearPendingIRQ(LPTIM_BSTIM_IRQn);/* 须保留LPTIM16的NVIC使能 */    

    FL_ADC_Disable(ADC);
    FL_ADC_DisableIT_EndOfSequence(ADC);
    FL_ADC_ClearFlag_EndOfSequence(ADC);
    Close_And_ClearIRQ(ADC_IRQn);
    
    FL_DMA_DisableChannel(DMA, FL_DMA_CHANNEL_4);
    FL_DMA_DisableIT_TransferComplete(DMA,FL_DMA_CHANNEL_4);      /* 禁能全程中断*/
    FL_DMA_DisableIT_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4);  /* 禁能半程中断*/ 
    FL_DMA_ClearFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4);     /* 清全程中断标志 */
    FL_DMA_ClearFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4); /* 清半程中断标志 */
    Close_And_ClearIRQ(DMA_IRQn);  
    
    /* 关闭非唤醒IO口 */
    CloseIO(GPIOD, FL_GPIO_PIN_0);
    CloseIO(GPIOD, FL_GPIO_PIN_1);
    CloseIO(GPIOD, FL_GPIO_PIN_2);
    CloseIO(GPIOD, FL_GPIO_PIN_9);
    CloseIO(GPIOD, FL_GPIO_PIN_10);
    CloseIO(GPIOD, FL_GPIO_PIN_14);
    CloseIO(GPIOD, FL_GPIO_PIN_15);
    CloseIO(GPIOF, FL_GPIO_PIN_6);
    CloseIO(GPIOF, FL_GPIO_PIN_7);
    
    /* 唤醒源配置 */
}

/**
  * @brief  ADC唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void adc_awake_init(uint8_t awake_type)
{
    /* 打开非唤醒IO */
    AnalogIO( GPIOD, FL_GPIO_PIN_0,  FL_GPIO_DRIVESTRENGTH_X3 );    //PD0;//ADC CH11
    AnalogIO( GPIOD, FL_GPIO_PIN_1,  FL_GPIO_DRIVESTRENGTH_X3 );    //PD1;//ADC CH12
    AnalogIO( GPIOD, FL_GPIO_PIN_2,  FL_GPIO_DRIVESTRENGTH_X3 );    //PD2;//ADC CH13
    AnalogIO( GPIOD, FL_GPIO_PIN_9,  FL_GPIO_DRIVESTRENGTH_X3 );    //PD9;//ADC CH0
    AnalogIO( GPIOD, FL_GPIO_PIN_10, FL_GPIO_DRIVESTRENGTH_X3 );    //PD10;//ADC CH1
    AnalogIO( GPIOD, FL_GPIO_PIN_14, FL_GPIO_DRIVESTRENGTH_X3 );    //PD14;//ADC CH5
    AnalogIO( GPIOD, FL_GPIO_PIN_15, FL_GPIO_DRIVESTRENGTH_X3 );    //PD15;//ADC CH6    
    AnalogIO( GPIOF, FL_GPIO_PIN_6,  FL_GPIO_DRIVESTRENGTH_X3 );    //PF6;//ADC CH7
    AnalogIO( GPIOF, FL_GPIO_PIN_7,  FL_GPIO_DRIVESTRENGTH_X3 );    //PF7;//ADC CH8  
    
    /* 任务初始化 */
    memset((void *)ADC_Result, 0, sizeof(ADC_Result));
    uState = FIRST_CH_SELECT;

    /* 唤醒源处理 */
    
    /* 非唤醒外设处理 */
    FL_ADC_ClearFlag_EndOfSequence(ADC);
    FL_ADC_EnableIT_EndOfSequence(ADC);
    NVIC_EnableIRQ(ADC_IRQn);
    FL_ADC_Enable(ADC);
    
    FL_DMA_ClearFlag_TransferComplete(DMA,FL_DMA_CHANNEL_4);     /* 清全程中断标志 */
    FL_DMA_ClearFlag_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4); /* 清半程中断标志 */
    FL_DMA_EnableIT_TransferComplete(DMA,FL_DMA_CHANNEL_4);      /* 禁能全程中断*/
    FL_DMA_EnableIT_TransferHalfComplete(DMA,FL_DMA_CHANNEL_4);  /* 禁能半程中断*/ 
    NVIC_EnableIRQ(DMA_IRQn);
    FL_DMA_WriteMemoryAddress(DMA, (uint32_t)((uint16_t *)(ADC_Result)), FL_DMA_CHANNEL_4);/* 重新配置ADDR，避免停留在上次停止位置 */
    FL_DMA_EnableChannel(DMA, FL_DMA_CHANNEL_4);
    
    FL_BSTIM32_ClearFlag_Update(BSTIM32);
    FL_BSTIM32_EnableIT_Update(BSTIM32);
    NVIC_EnableIRQ(LPTIM_BSTIM_IRQn); 
    FL_BSTIM32_Enable(BSTIM32);  /* 使能定时器 *///TODO 需要验证关闭外设时机
    
    FL_BSTIM16_ClearFlag_Update(BSTIM16);    
    FL_BSTIM16_EnableIT_Update(BSTIM16);
    NVIC_EnableIRQ(LPTIM_BSTIM_IRQn);
//    FL_BSTIM16_Enable(BSTIM16);  /* 使能定时器 */

    /* 打开Buffer功能 */
    FL_PMU_EnableVPTATBuffer(PMU);                      /* 使能VPTAT Buffer */
    FL_PMU_EnableAVREFBuffer(PMU);                      /* 使能AVREF BUFFER  */
}

/**
  * @brief  ADC sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t adc_is_sleep(void)
{
    if (!FL_ADC_IsActiveFlag_Busy(ADC))
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif
