 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "flash.h"
#include "svd.h"
#include "mf_config.h"
#include "verify.h"
#include <string.h>

/* Code Flash擦写起始地址 */
#define CODE_FLASH_PROG_ADDR    0x00034000U

/* Data Flash擦写起始地址，建议应用保留第0扇区 */
#define DATA_FLASH_PROG_ADDR    0xA0000800U


#define BUFFER_SIZE             512U
uint32_t DataBuffer[BUFFER_SIZE];

/**
  * @brief  Code Flash 擦写示例
  * @param  void
  * @retval None
*/
void CodeFlash_RW(void)
{
    /* 判断MCU电源，避免在擦写时刻发生BOR掉电复位 */
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_CodePageErase(FLASH, CODE_FLASH_PROG_ADDR);
        memset(DataBuffer, 0x55, BUFFER_SIZE * 4);
        (void)FL_FLASH_CodeProgram_Word(FLASH, CODE_FLASH_PROG_ADDR, DataBuffer[0]);
    }
    
    /* 判断MCU电源，避免在擦写时刻发生BOR掉电复位 */
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_CodePageErase(FLASH, CODE_FLASH_PROG_ADDR);
        memset(DataBuffer, 0xAA, BUFFER_SIZE * 4);
        (void)FL_FLASH_CodeProgram_Page(FLASH, CODE_FLASH_PROG_ADDR / FL_FLASH_CODE_PGAE_SIZE_BYTE, DataBuffer);
    }
    
    /* 判断MCU电源，避免在擦写时刻发生BOR掉电复位 */
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_CodeSectorErase(FLASH, CODE_FLASH_PROG_ADDR);
        memset(DataBuffer, 0x00, BUFFER_SIZE * 4);
        (void)FL_FLASH_CodeProgram_Sector(FLASH, CODE_FLASH_PROG_ADDR / FL_FLASH_CODE_SECTOR_SIZE_BYTE, DataBuffer);
    }
    
    /* 判断MCU电源，避免在擦写时刻发生BOR掉电复位 */ 
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_CodeSectorErase(FLASH, CODE_FLASH_PROG_ADDR);
    }
}


/**
  * @brief  Data Flash 擦写示例
  * @param  void
  * @retval None
*/
void DataFlash_RW(void)
{
    /* 判断MCU电源，避免在擦写时刻发生BOR掉电复位 */
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_DataPageErase(FLASH, DATA_FLASH_PROG_ADDR);
        memset(DataBuffer, 0x55, BUFFER_SIZE * 4);
        (void)FL_FLASH_DataProgram_Word(FLASH, DATA_FLASH_PROG_ADDR, DataBuffer[0]);
    }
    
    /* 判断MCU电源，避免在擦写时刻发生BOR掉电复位 */
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_DataPageErase(FLASH, DATA_FLASH_PROG_ADDR);
        memset(DataBuffer, 0xAA, BUFFER_SIZE * 4);
        (void)FL_FLASH_DataProgram_Page(FLASH, (DATA_FLASH_PROG_ADDR - FL_FLASH_DATA_ADDR_MINPROGRAM) / FL_FLASH_DATA_PGAE_SIZE_BYTE, DataBuffer);
    }
    
    /* 判断MCU电源，避免在擦写时刻发生BOR掉电复位 */
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_DataSectorErase(FLASH, DATA_FLASH_PROG_ADDR);
        memset(DataBuffer, 0x00, BUFFER_SIZE * 4);
        (void)FL_FLASH_DataProgram_Sector(FLASH, (DATA_FLASH_PROG_ADDR - FL_FLASH_DATA_ADDR_MINPROGRAM) / FL_FLASH_DATA_SECTOR_SIZE_BYTE, DataBuffer);
    }
    
    /* 判断MCU电源，避免在擦写时刻发生BOR掉电复位 */ 
    if (true == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        (void)FL_FLASH_DataSectorErase(FLASH, DATA_FLASH_PROG_ADDR);
    }
}

uint8_t Write_DataFlash(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)
{
    uint16_t h, i, k, status, result;
    static uint8_t BufPageBKUP[FL_FLASH_DATA_PGAE_SIZE_BYTE] __attribute__((aligned(4)));/* 回读当前Page内容 4字节对齐 */
    uint8_t BufA[FL_FLASH_DATA_PGAE_SIZE_BYTE];
    uint32_t PageStartAddress;

    if (Page >= FL_FLASH_DATA_MAX_PAGE_NUM) return 1;  //地址溢出
    
    h = Len;
    memcpy(BufA, Buf, Len);
    
    if (Page < DATA_FLASH_CRC_PAGE)
    {
        Crc_16Bits_Add2(BufA, Len);          //计算crc
        h = h + 2;                  //长度加2字节crc
    }
    if ((offset + h) > FL_FLASH_DATA_PGAE_SIZE_BYTE) return 1; //空间溢出
    
    PageStartAddress = FL_FLASH_DATA_ADDR_MINPROGRAM + Page * FL_FLASH_DATA_PGAE_SIZE_BYTE;
    
    /* 判断MCU电源，避免在擦写时刻发生BOR掉电复位 */
    if (true != SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 1U))
    {
        return 2;
    }
    
    memcpy(BufPageBKUP, (void *)(PageStartAddress), FL_FLASH_DATA_PGAE_SIZE_BYTE);/* 先回读 */
    memcpy(&BufPageBKUP[offset], BufA, h);/* 更新要写入部分 */

    for (i = 0; i < 2; i++) //最多写2遍
    {
        for (k = 0; k < 3; k++) //每遍最多写3次
        {
            status = 0;
            result = FL_FLASH_DataPageErase(FLASH, FL_FLASH_DATA_ADDR_MINPROGRAM + Page * FL_FLASH_DATA_PGAE_SIZE_BYTE);
            if (result != FL_PASS) status = 1;  //擦除失败
            else
            {
                result = FL_FLASH_DataProgram_Page(FLASH, Page, (uint32_t *)BufPageBKUP);
                if (result != FL_PASS)
                {
                    status = 1;
                }
            }
            
            if (status == 0)
            {
                break;
            }
        }
        if (status == 1) continue; //错误结束本次读

        //--------------------------------- verify -------------------------------------
        result = memcmp(BufPageBKUP, (void *)(PageStartAddress), FL_FLASH_DATA_PGAE_SIZE_BYTE);
        if (result == 0)
        {
            if (Page < DATA_FLASH_CRC_PAGE)/* 更新CRC数值 */
            {
                memcpy(Buf, BufA, h);
            }
            return 0;
        }
    }
    return 3; //fail
}

uint8_t Read_DataFlash(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)
{
    uint16_t h;

    h = Len;
    if (Page < DATA_FLASH_CRC_PAGE) h = h + 2;      //长度加2字节crc

    if ((offset + h) > FL_FLASH_DATA_PGAE_SIZE_BYTE) return 1; //空间溢出
    
    memcpy(Buf, (void *)(FL_FLASH_DATA_ADDR_MINPROGRAM + Page * FL_FLASH_DATA_PGAE_SIZE_BYTE + offset), h);
    return 0;
}

uint8_t WriteDataFlashWithBackup(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)
{
    if (Write_DataFlash(Page, offset, Buf, Len)) return 1;

    Write_DataFlash(Page + DATA_FLASH_BKUP_PAGE, offset, Buf, Len);

    return 0;
}

void  ReadDataFlashWithBackup(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)
{
    if (Read_DataFlash(Page, offset, Buf, Len))
    {
        Read_DataFlash(Page + DATA_FLASH_BKUP_PAGE, offset, Buf, Len);
    }
}

