 /**    
  *******************************************************************************************************    
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "i2c.h"
#include "verify.h"
#include "mf_config.h"
#include <string.h>

#define I2CREAD       1     /* I2C读操作 */
#define I2CWRITE      0     /* I2C写操作 */

#define  I2C_TIMEOUT  0x7FFFFU  /* I2C超时时间 */

/* 信号类型 */
#define STARTBIT      0
#define RESTARTBIT    1
#define STOPBIT       2

#define DEVICE_EE256  0xA0  /* FM24C256的器件地址 */
#define PAGE_SIZE     64    /* page size */
#define ADDRLEN_EE256 2     /* FM24C256的数据地址长度为2字节 */



/**
  * @brief  I2C Initialization function
  * @param  void
  * @retval None
  */
void I2C_Init(void)
{
    FL_GPIO_InitTypeDef    GPIO_InitStruct;
    FL_I2C_MasterMode_InitTypeDef    defaultInitStruct;
  
    /* 初始化引脚功能 */
    GPIO_InitStruct.pin = FL_GPIO_PIN_3;                       /*配置GPIO的引脚号*/
    GPIO_InitStruct.mode = FL_GPIO_MODE_DIGITAL;               /*配置GPIO的功能模式*/
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_OPENDRAIN;     /*配置GPIO的输出模式*/
    GPIO_InitStruct.pull = FL_GPIO_BOTH_DISABLE;               /*配置GPIO上拉下拉模式*/
    GPIO_InitStruct.remapPin = FL_GPIO_PINREMAP_FUNCTON0;      /*配置GPIO数字重定向功能*/
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;  /*配置GPIO驱动能力*/
    (void)FL_GPIO_Init(GPIOH, &GPIO_InitStruct);               /*GPIO初始化*/

    GPIO_InitStruct.pin = FL_GPIO_PIN_4;                       /*配置GPIO的引脚号*/
    GPIO_InitStruct.mode = FL_GPIO_MODE_DIGITAL;               /*配置GPIO的功能模式*/
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_OPENDRAIN;     /*配置GPIO的输出模式*/
    GPIO_InitStruct.pull = FL_GPIO_BOTH_DISABLE;               /*配置GPIO上拉下拉模式*/
    GPIO_InitStruct.remapPin = FL_GPIO_PINREMAP_FUNCTON0;      /*配置GPIO数字重定向功能*/
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;  /*配置GPIO驱动能力*/
    (void)FL_GPIO_Init(GPIOH, &GPIO_InitStruct);               /*GPIO初始化*/

    defaultInitStruct.baudRate = I2C_BAUDRATE;                       /*I2C波特率设置*/
    (void)FL_I2C_MasterMode_Init(I2C1, &defaultInitStruct);    /*I2C初始化*/
}



/**
  * @brief  发送时序
  * @param  I2Cx I2C入口地址
  * @param  BIT_def 时序，包括STARTBIT,RESTARTBIT,STOPBIT
  * @retval result = 1 发送未完成
  * @retval result = 0 发送完成
  */
static uint8_t I2C_Send_Bit(I2C_Type *I2Cx, uint8_t BIT_def)
{
    uint8_t result = 1;
    uint32_t Start_Flag, Stop_Flag, Restart_Flag;
    uint32_t Counter;
    switch(BIT_def)
    {
        case STARTBIT:
            FL_I2C_Master_EnableI2CStart(I2Cx);  /* 产生start信号 */
            
            Counter = 0;
            do 
            {
                Start_Flag = FL_I2C_Master_IsActiveFlag_Start(I2Cx);
                Counter++;
            }while((Counter != I2C_TIMEOUT) && (Start_Flag == 0U));
            if(Start_Flag == 0x01)
            {
                result = 0;
            }
            else
            {
                result = 1;
            }
            break;
        
        case RESTARTBIT:
            FL_I2C_Master_EnableI2CRestart(I2Cx);  /* 产生restart信号 */
            
            Counter = 0;
            do
            {
                Restart_Flag = FL_I2C_Master_IsActiveFlag_Start(I2Cx);
                Counter++;
            } while((Counter != I2C_TIMEOUT) && (Restart_Flag == 0U));  /* 等待转换完成 */

            if(Restart_Flag == 0x01)
            {
                result = 0;
            }
            else
            {
                result = 1;
            }
            break;
        
        case STOPBIT:
            FL_I2C_Master_EnableI2CStop(I2Cx);  /* 产生stop信号 */
           
            Counter = 0;
            do
            {
                Stop_Flag = FL_I2C_Master_IsActiveFlag_Stop(I2Cx);
                Counter++;
            } while((Counter != I2C_TIMEOUT) && (Stop_Flag == 0U));  /* 等待转换完成 */

            if(Stop_Flag == 0x01)
            {
                result = 0;
            }
            else
            {
                result = 1;
            }
            break;
        default:
            result = 1;
            break;
    }
    return result; /* 返回状态标志 */
}


/**
  * @brief  发送字节数据
  * @param  I2Cx I2C入口地址
  * @param  x_byte 数据，一个字节
  * @retval result = 1 发送未完成
  * @retval result = 0 发送完成
  */
static uint8_t I2C_Send_Byte(I2C_Type *I2Cx, uint8_t x_byte)
{
    uint8_t result = 1;
    uint32_t TX_Flag;
    uint32_t Counter;

    FL_I2C_Master_WriteTXBuff(I2Cx, x_byte);  /* 发送数据 */
  
    Counter = 0;
    do
    {
        TX_Flag = FL_I2C_Master_IsActiveFlag_TXComplete(I2Cx);
        Counter++;
    } while((Counter != I2C_TIMEOUT) && (TX_Flag == 0U));  /* 等待转换完成 */
    
    if(TX_Flag == 0x01)
    {
        FL_I2C_Master_ClearFlag_TXComplete(I2Cx);
    }
    else
    {
        return 1;
    }
    
    if(FL_I2C_Master_IsActiveFlag_NACK(I2Cx) == 0)  /* 判断接收到ACK or NACK */
    {
        result = 0;
    }
    else
    {
        FL_I2C_Master_ClearFlag_NACK(I2Cx);
        result = 1;
    }

    return result; /* 返回状态标志 */
}


/**
  * @brief  接收字节数据
  * @param  I2Cx I2C入口地址
  * @param  *x_byte 接收数据地址
  * @retval result = 1 接收未完成
  * @retval result = 0 接收完成
  */
static uint8_t I2C_Receive_Byte(I2C_Type *I2Cx, uint8_t *x_byte)
{
    uint32_t Counter;
    uint32_t RX_Flag;
    uint8_t result = 1;
    FL_I2C_Master_EnableRX(I2Cx);  /* 配置接收使能 */

    Counter = 0;
    do
    {
        RX_Flag = FL_I2C_Master_IsActiveFlag_RXComplete(I2Cx);
        Counter++;
    } while((Counter != I2C_TIMEOUT) && (RX_Flag == 0U));  /* 等待接收完成 */

    if(RX_Flag == 0x01)
    {
        FL_I2C_Master_ClearFlag_RXComplete(I2Cx);
    }
    else
    {
        return 1;
    }

    *x_byte = (uint8_t)(FL_I2C_Master_ReadRXBuff(I2Cx) & 0xFFU);
    result  = 0;

    return result;    /* 返回状态标志 */
}


/**
  * @brief  发送地址
  * @param  I2Cx I2C入口地址
  * @param  Device 器件地址
  * @param  Addr 数据地址
  * @param  AddrLen 数据地址长度
  * @param  Opt 操作类型
  * @retval result = 1 接收未完成
  * @retval result = 0 接收完成
  */
static uint8_t Sendaddr(I2C_Type *I2Cx, uint8_t Device, uint16_t Addr, uint8_t AddrLen, uint8_t Opt)
{
    uint8_t result, Devi_Addr;

    Devi_Addr = Device;

    /*-------------- start bit ---------------*/
    result = I2C_Send_Bit(I2Cx, STARTBIT);                    /* 发送起始位 */

    if (result != 0)
    {
        return 1;    /* failure. */
    }

    /*-------------- disable read -------------*/
    FL_I2C_Master_DisableRX(I2Cx);
    /*-------------- device addr -------------*/
    result = I2C_Send_Byte(I2Cx, Devi_Addr);                  /* 发送器件地址 */

    if (result != 0)
    {
        return 2;    /* failure. */
    }

    /*--------------- data addr --------------*/
    if (AddrLen == 2)
    {
        result = I2C_Send_Byte(I2Cx, (uint8_t)(Addr >> 8U));  /* 发送数据地址高8位 */

        if (result != 0)
        {
            return 3;    /* failure. */
        }
    }

    result = I2C_Send_Byte(I2Cx, (uint8_t)(Addr >> 0U));      /* 发送数据地址低8位 */

    if (result != 0)
    {
        return 3;    /* failure. */
    }

    if (Opt == I2CREAD)                                       /* 判断是否是接收逻辑 */
    {
        result = I2C_Send_Bit(I2Cx, RESTARTBIT);              /* 发送重起始位 */

        if (result != 0)
        {
            return 5;    /* failure. */
        }

        result = I2C_Send_Byte(I2Cx, Devi_Addr | 0x01U);      /* 发送器件地址（读逻辑） */

        if (result != 0)
        {
            return 5;    /* failure. */
        }
    }

    return 0; /* ok */
}

/**
  * @brief  等待结束
  * @param  I2Cx I2C入口地址
  * @param  Device 器件地址
  * @retval result = 1 超时
  * @retval result = 0 未超时
  */
static uint8_t Wait_for_end(I2C_Type *I2Cx, uint8_t Device)
{
    uint8_t result, Devi_Addr;
    uint32_t counter = 0;
    Devi_Addr = Device;
    do
    {
        I2C_Send_Bit(I2Cx, STARTBIT);             /* 发送起始位 */
        result = I2C_Send_Byte(I2Cx, Devi_Addr);  /* 发送器件地址 */
        I2C_Send_Bit(I2Cx, STOPBIT);              /* 发送停止位 */
        counter++;
    } while((result == 0x01U) && (counter != I2C_TIMEOUT));

    if(result == 0)  /* 设置地址成功, 写周期结束  */
    {
        return 0;
    }
    else
    {
        return 1;    /* 设置地址失败,超时 */
    }
}

/**
  * @brief  写入数据
  * @param  I2Cx I2C入口地址
  * @param  Device 器件地址
  * @param  Addr 数据地址
  * @param  AddrLen 数据地址长度
  * @param  Buf 写入数据地址
  * @param  Len 写入数据长度
  * @retval result = 1 接收未完成
  * @retval result = 0 接收完成
  */
static uint8_t I2C_Write_Bottom(I2C_Type *I2Cx, uint8_t Device, uint16_t Addr, uint8_t AddrLen, uint8_t *Buf, uint8_t Len)
{
    uint8_t k, n, status;

    for(k = 0; k < 3; k++)                             /* 每遍最多写3次  */
    {
        status = 0;

        if(Sendaddr(I2Cx, Device, Addr, AddrLen, I2CWRITE) == 1U)
        {
            status = 1;                                /* 写入失败 */
        }
        else
        {
            for(n = 0; n < Len; n++)
            {

                if(I2C_Send_Byte(I2Cx, Buf[n]) == 1U)  /* 发送一个字节 */
                {
                    status = 1;                        /* 写入失败 */
                    break;
                }
            }
        }


        if(I2C_Send_Bit(I2Cx, STOPBIT) == 1)           /* 发送停止位  */
        {
            status = 1;
        }

        if(status == 0)
        {
            status = Wait_for_end(I2Cx, Device);       /* 写正确 */
            break;
        }
    }

    return(status);
}

/**
  * @brief  读取数据
  * @param  I2Cx I2C入口地址
  * @param  Device 器件地址
  * @param  Addr 数据地址
  * @param  AddrLen 数据地址长度
  * @param  Buf 数据保存地址
  * @param  Len 数据保存长度
  * @retval result = 1 接收未完成
  * @retval result = 0 接收完成
  */
static uint8_t I2C_Read_Bottom(I2C_Type *I2Cx, uint8_t Device, uint16_t Addr, uint8_t AddrLen, uint8_t *Buf, uint8_t Len)
{
    uint8_t k, n, status;

    for(k = 0; k < 3; k++)  /* 每遍最多写3次 */
    {
        status = 0;

        if(Sendaddr(I2Cx, Device, Addr, AddrLen, I2CREAD) == 1U)
        {
            status = 1;     /* 写入失败 */
        }
        else
        {
            for(n = 0; n < Len; n++)
            {
                if(n < (Len - 1))
                {
                    FL_I2C_Master_SetRespond(I2Cx, FL_I2C_MASTER_RESPOND_ACK);
                }
                else
                {
                    FL_I2C_Master_SetRespond(I2Cx, FL_I2C_MASTER_RESPOND_NACK);
                }

                (void)I2C_Receive_Byte(I2Cx, Buf + n);  /* 接受一个字节 */
            }
        }

        if(I2C_Send_Bit(I2Cx, STOPBIT) == 1U)  /* 发送停止位 */
        {
            status = 1;                        /* 失败 */
        }

        if(status == 0)
        {
            break;                             /* 读正确 */
        }
    }

    return(status);
}

uint8_t I2C_Write_Eeprom(I2C_Type *I2Cx, uint32_t Page, uint8_t offset, uint8_t *IIC_Buf, uint8_t Len)
{
    uint8_t h, i, k, n, status, x_byte, result;
    uint8_t BufA[PAGE_SIZE];

    if (Page >= 0x200) return 1;  //地址溢出
    h = Len;
    memcpy(BufA, IIC_Buf, Len);
    if (Page < 0x56)
    {
        Crc_16Bits_Add2(BufA, Len);          //计算crc
        h = h + 2;                  //长度加2字节crc
    }
    if ((offset + h) > PAGE_SIZE) return 1; //空间溢出

    for (i = 0; i < 2; i++) //最多写3遍
    {
        for (k = 0; k < 3; k++) //每遍最多写3次
        {
            status = 0;
            result = Sendaddr(I2Cx, DEVICE_EE256, Page*PAGE_SIZE+offset, ADDRLEN_EE256, I2CWRITE);
            if (result != 0) status = 1;  //写入失败
            else
            {
                for (n = 0; n < h; n++)
                {
                    result = I2C_Send_Byte(I2Cx, BufA[n]);
                    if (result != 0)
                    {
                        status = 1;    //写入失败
                        break;
                    }
                }
            }
            result = I2C_Send_Bit(I2Cx, STOPBIT);   //发送停止位
            if (result != 0) status = 1;
            if (status == 0)
            {
                Wait_for_end(I2Cx, DEVICE_EE256);    //写正确
                break;
            }
        }
        if (status == 1) continue; //错误结束本次读

        //--------------------------------- verify -------------------------------------
        for (k = 0; k < 3; k++) //每遍最多读3次
        {
            status = 0;
            result = Sendaddr(I2Cx, DEVICE_EE256, Page*PAGE_SIZE+offset, ADDRLEN_EE256, I2CREAD);
            if (result != 0) status = 1;  //写入失败
            else
            {
                for (n = 0; n < h; n++)
                {
                    if (n < (h - 1))
                    {
                        FL_I2C_Master_SetRespond(I2Cx, FL_I2C_MASTER_RESPOND_ACK);
                    }
                    else
                    {
                        FL_I2C_Master_SetRespond(I2Cx, FL_I2C_MASTER_RESPOND_NACK);
                    }
                    
                    result = I2C_Receive_Byte(I2Cx, &x_byte);  /* 接收一个字节 */
                    if (result != 0)
                    {
                        status = 1;
                        break;
                    }

                    if (x_byte != BufA[n])
                    {
                        status = 1;    //读出的数据与写入的数据不符, 退出读取程序
                        break;
                    }
                }
            }
            result = I2C_Send_Bit(I2Cx, STOPBIT);   //发送停止位
            if (result != 0) status = 1;
            if (status == 0) return 0; //写成功
        }
    }
    return 2; //fail
}

uint8_t I2C_Read_Eeprom(I2C_Type *I2Cx, uint32_t Page, uint8_t offset, uint8_t *IIC_Buf, uint8_t Len)
{
    uint8_t h, i, k, n, status, result;
    uint8_t BufA[PAGE_SIZE];

    h = Len;
    if (Page < 0x56) h = h + 2;      //长度加2字节crc

    if ((offset + h) > PAGE_SIZE) return 1; //空间溢出

    for (i = 0; i < 2; i++) //最多读3遍
    {
        for (k = 0; k < 3; k++) //每遍最多读3次
        {
            status = 0;
            result = Sendaddr(I2Cx, DEVICE_EE256, Page*PAGE_SIZE+offset, ADDRLEN_EE256, I2CREAD);
            if (result != 0) status = 1;  //写入失败
            else
            {
                for (n = 0; n < h; n++)
                {
                    if (n < (h - 1))
                    {
                        FL_I2C_Master_SetRespond(I2Cx, FL_I2C_MASTER_RESPOND_ACK);
                    }
                    else
                    {
                        FL_I2C_Master_SetRespond(I2Cx, FL_I2C_MASTER_RESPOND_NACK);
                    }
                    
                    result = I2C_Receive_Byte(I2Cx, &IIC_Buf[n]);   //接收一个字节
                    if (result != 0)
                    {
                        status = 1;
                        break;
                    }
                }
            }
            result = I2C_Send_Bit(I2Cx, STOPBIT);   //发送停止位
            if (result != 0) status = 1;
            if (status == 0) break; //读正确
        }
        if (status == 1) continue; //错误结束本次读


        for (k = 0; k < 3; k++) //每遍最多读3次
        {
            status = 0;
            result = Sendaddr(I2Cx, DEVICE_EE256, Page*PAGE_SIZE+offset, ADDRLEN_EE256, I2CREAD);
            if (result != 0) status = 1;  //写入失败
            else
            {
                for (n = 0; n < h; n++)
                {
                    if (n < (h - 1))
                    {
                        FL_I2C_Master_SetRespond(I2Cx, FL_I2C_MASTER_RESPOND_ACK);
                    }
                    else
                    {
                        FL_I2C_Master_SetRespond(I2Cx, FL_I2C_MASTER_RESPOND_NACK);
                    }
                    
                    result = I2C_Receive_Byte(I2Cx, &BufA[n]);   //接收一个字节
                    if (result != 0)
                    {
                        status = 1;
                        break;
                    }
                }
            }
            result = I2C_Send_Bit(I2Cx, STOPBIT);   //发送停止位
            if (result != 0) status = 1;
            if (status == 0) break; //读正确
        }
        if (status == 1) continue; //错误结束本次读

        //--------------------------------- verify -------------------------------------
        for (n = 0; n < h; n++)
        {
            if (BufA[n] != IIC_Buf[n]) break;
        }
        if (n != h) continue; //比较不一致错误

        if (Page >= 0x56) return 0; //读成功

        if (Crc_16Bits(IIC_Buf, h) == 0) return 0;   //读成功
    }
    return 2; //fail
}


void  ReadE2WithBackup(I2C_Type *I2Cx, uint32_t Page, uint8_t *Buf, uint8_t Len)
{
    if (I2C_Read_Eeprom(I2Cx, Page, 0x00, Buf, Len))
    {
        I2C_Read_Eeprom(I2Cx, Page, 0x20, Buf, Len);
    }
}


uint8_t WriteE2WithBackup(I2C_Type *I2Cx, uint32_t Page, uint8_t *Buf, uint8_t Len)
{
    if (I2C_Write_Eeprom(I2Cx, Page, 0x00, Buf, Len)) return 1;

    I2C_Write_Eeprom(I2Cx, Page, 0x20, Buf, Len);

    return 0;
}

/**
  * @brief  I2C读写EEPROM
  * @param  I2Cx I2C入口地址
  * @retval result = 1 操作未完成
  * @retval result = 0 操作完成
  */
uint8_t I2CWriteAndRead(void)
{
    uint8_t Device = DEVICE_EE256;
    uint16_t Addr;
    uint8_t AddrLen = ADDRLEN_EE256;
    uint8_t Buf[2] = {0, 0};
    uint8_t Len = 0;
    uint8_t Write_Result = 0, Read_Result = 0;
    uint8_t Result = 1;
    uint8_t TestData[2] = {0xAA, 0x55};

    Addr = 10 * PAGE_SIZE;
    Len = 2;

    FL_IWDT_ReloadCounter(IWDT);
    Write_Result = I2C_Write_Bottom(I2C1, Device, Addr, AddrLen, TestData, Len); /* 写EE */
    FL_IWDT_ReloadCounter(IWDT);
    Read_Result = I2C_Read_Bottom(I2C1, Device, Addr, AddrLen, Buf, Len);        /* 读EE */
    if ((Write_Result == 0) && (Read_Result == 0))
    {
        Result = 0;
    }
    return Result;
}

#if (SLEEP_SUPPORT == 1)
/**
  * @brief  I2C1 休眠前初始化
  * @param  void
  * @retval void
  */
void i2c1_sleep_init(void)
{
    /* 关闭非唤醒外设 */
    FL_I2C_Master_Disable(I2C1);
    
    /* 关闭非唤醒IO口 */
    CloseIO(GPIOH, FL_GPIO_PIN_3);
    CloseIO(GPIOH, FL_GPIO_PIN_4);
    
    /* 唤醒源配置 */
}

/**
  * @brief  I2C1 唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void i2c1_awake_init(uint8_t awake_type)
{
    /* 打开非唤醒IO */
    AltFunIO( GPIOH, FL_GPIO_PIN_3,  FL_GPIO_OUTPUT_OPENDRAIN, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    //PH3;//I2C1 SCL
    AltFunIO( GPIOH, FL_GPIO_PIN_4,  FL_GPIO_OUTPUT_OPENDRAIN, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    //PH4;//I2C1 SDA    
    
    /* 任务初始化 */

    /* 唤醒源处理 */
    
    /* 非唤醒外设处理 */
    FL_I2C_Master_Enable(I2C1);   
}

/**
  * @brief  I2C1 sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t i2c1_is_sleep(void)
{
    if (!FL_I2C_Master_IsActiveFlag_Busy(I2C1))
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif
