 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "main.h"
#include "lin.h"
#include "uart.h"
#include "sleep.h"
#include "clock.h"
#include "svd.h"
#include "led.h"
#include "svd_interrupt.h"
#include "mf_config.h"
#include "i2c.h"

#define FRAME_DATA_MAX_LEN          11  /* 帧最大数据长度 */
#define SCHEDULE_FRAME_MAX_NUM      255 /* 进度表最大帧数量 */

#define FRAME_TYPE_UNCONDITIONAL    0   /* 无条件帧 */
#define FRAME_TYPE_EVENT_TRIGGER    1   /* 事件触发帧 */
#define FRAME_TYPE_SPORADIC         2   /* 偶发帧 */
#define FRAME_TYPE_DIAGNOSTIC       3   /* 诊断帧 */
#define FRAME_TYPE_RESERVED         4   /* 保留帧 */
#define FRAME_TYPE_NO_RESPONSE      5   /* 自定义，主机发送无需回应帧 */

/* 帧处理状态机 */
#define FRAME_STATE_IDLE            0   /* 空闲状态 主：等待任务开始 从：固定波特率：等待间隔段 自动波特率：等待间隔段+同步段 */
#define FRAME_STATE_SEND_SYN        1   /* 主机发送SYN(包括间隔段和同步段) */
#define FRAME_STATE_RECV_SYN        2   /* 固定波特率：接收同步段0x55 */
#define FRAME_STATE_RECV_PID        3   /* 接收PID */
#define FRAME_STATE_RECV_DATA       4   /* 接收数据 */
#define FRAME_STATE_SEND            5   /* 串口发送 */
#define FRAME_STATE_SLEEP           6   /* 休眠状态 */

#if (SLEEP_SUPPORT == 1)
extern uint8_t  g_master_awake_init;
extern uint8_t  g_slave_awake_init;

void lin_master_sleep_init(void);
void lin_slave_sleep_init(void);
#endif

/**
  * @brief  计算PID
  * @param  id: 输入id
  * @retval 计算后的PID 
  */
uint8_t calculatePID(uint8_t id)
{
    uint8_t P0 = 0, P1 = 0;

    P0 = ((id) ^ (id >> 1) ^ (id >> 2) ^ (id >> 4)) & 0x01;
    P1 = (~((id >> 1) ^ (id >> 3) ^ (id >> 4) ^ (id >> 5))) & 0x01;

    return ((P1 << 7) | (P0 << 6) | id);
}

/**
  * @brief  计算校验和
  * @param  data: 数据指针
  *         len:  数据长度
  * @retval checkSum: 校验和 
  */
uint8_t calculateChecksum(uint8_t *data, uint8_t len)
{
    uint8_t i;
    uint16_t checkSum = 0;

    for (i = 0; i < len; i++)
    {
        checkSum += data[i];
        if (checkSum >= 0x100)
        {
            checkSum -= 0xFF;
        }
    }
    checkSum = (~checkSum) & 0xFF;

    return checkSum;
}

/**
  * @brief  LIN Master slp初始化
  * @param  void
  * @retval void
  */
void lin_master_slp_pin_init(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};

    LIN_MASTER_SLP_DISABLE();

    GPIO_InitStruct.pin           = LIN_MASTER_SLP_PIN;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull          = FL_GPIO_BOTH_DISABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON0;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    (void)FL_GPIO_Init(LIN_MASTER_SLP_GPIO, &GPIO_InitStruct);
}

/**
  * @brief  LIN Slave slp初始化
  * @param  void
  * @retval void
  */
void lin_slave_slp_pin_init(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};

    LIN_SLAVE_SLP_DISABLE();

    GPIO_InitStruct.pin           = LIN_SLAVE_SLP_PIN;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull          = FL_GPIO_BOTH_DISABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON0;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    (void)FL_GPIO_Init(LIN_SLAVE_SLP_GPIO, &GPIO_InitStruct);
}


#define Min(a, b) (((a)<(b))?(a):(b))
#define lin_calc_max_res_timeout_cnt(bitLength, size) (uint16_t)((14*(1+(size))*((bitLength)/TIMER_1US_LOAD))/TIME_BASE_PERIOD + 3 )

/****************************************  主机  ************************************/

/* 进度表帧格式 [最大11字节] */
/* 同步段0x55-1字节；受保护ID-1字节; 数据段-0-8字节；校验和-1字节 */
typedef struct m_lin_frame_type
{
    uint16_t frame_slot;           /* 帧时隙，LIN子网时基的整数倍 */
    uint16_t frame_left_slot;      /* 剩余slot */
    uint8_t frame_type;            /* 帧类型 */
    uint8_t timeout;               /* 应答超时 0-无应答超时 其余-ms */
    uint8_t tx_len;                /* 发送长度，至少包括2字节 */
    uint8_t rx_len;                /* 接收长度 */
    uint8_t data[FRAME_DATA_MAX_LEN];
}lin_schedule_data_type;

/* 进度表，最大支持255帧 */
typedef struct s_lin_schedule_type
{
    uint8_t frame_num;              /* 帧数目 */
    lin_schedule_data_type lin_schedule_data[SCHEDULE_FRAME_MAX_NUM];
}lin_schedule_type;

/* 任务处理：循环发送 */
typedef struct m_lin_task_type
{
    uint8_t frame_index;            /* 当前进度表帧索引值 */
    uint8_t frame_state;            /* 帧处理状态机，参照 FRAME_STATE_INIT 定义 */
    
    /* 帧超时计数，接收到同步段后置起，按照8字节数据计算超时 */
    /* 公式：round((1.4x(10+8x10)xTbit)/Tbase_period) + 3，8为接收字节数，Tbit为单个bit用时，Tbase_period为定时器周期，单位都是us */
    uint16_t frame_timeout_cnt;
    /* 响应超时计数，接收到需要响应的ID后置起，按照实际接收长度计算超时 */
    /* 公式：round((1.4x(10+Nx10)xTbit)/Tbase_period) + 3，N为接收字节数，Tbit为单个bit用时，Tbase_period为定时器周期，单位都是us */
    uint16_t res_frame_timeout_cnt;    
    lin_schedule_type lin_schedule; /* 进度表 */
}lin_master_task_type;

uint16_t schedule_master_table_slot[100];
lin_master_task_type lin_master_task;

uint8_t LIN_MASTER_CMD0_Data[8] = {0x02, 0x00, 0x00};
uint8_t LIN_MASTER_CMD1_Data[8];
uint8_t LIN_MASTER_CMD2_Data[8];

/* Master 进度表 */
lin_master_cmd_packet_t master_scheduleTable[] = {
  /* 用户命令码 */
  {LIN_MASTER_CMD0, TRANSMIT, LIN_MASTER_CMD0_LENGTH, LIN_MASTER_CMD0_TIMEOUT, LIN_MASTER_CMD0_PERIOD, LIN_MASTER_CMD0_Data},
  {LIN_MASTER_CMD1, RECEIVE,  LIN_MASTER_CMD1_LENGTH, LIN_MASTER_CMD1_TIMEOUT, LIN_MASTER_CMD1_PERIOD, LIN_MASTER_CMD1_Data},
  {LIN_MASTER_CMD2, RECEIVE,  LIN_MASTER_CMD2_LENGTH, LIN_MASTER_CMD2_TIMEOUT, LIN_MASTER_CMD2_PERIOD, LIN_MASTER_CMD2_Data},
};

/****************************************  从机  ************************************/
#if (AUTOBAUD == 1)


/* lin波特率和字节间间隔 */
#define LIN_BAUDRATE_MAX                20000  /* Lin支持的最大波特率 */
#if (MAIN_CLOCK == 16000000)                   /* 16MHz下对应BGR */
#define LIN_BAUDRATE_9600_BGR           1666                            
#define LIN_BAUDRATE_19200_BGR          832                             
#define LIN_SYN_BAUDRATE_MAX_BGR        39     /* 400K  */
#elif (MAIN_CLOCK == 24000000)                 /* 24MHz下对应BGR */
#define LIN_BAUDRATE_9600_BGR           2499 
#define LIN_BAUDRATE_19200_BGR          1249                            
#define LIN_SYN_BAUDRATE_MAX_BGR        59     /* 400K  */
#elif (MAIN_CLOCK == 48000000)                 /* 48MHz下对应BGR */
#define LIN_BAUDRATE_9600_BGR           4999 
#define LIN_BAUDRATE_19200_BGR          2499                            
#define LIN_SYN_BAUDRATE_MAX_BGR        119     /* 400K  */
#elif (MAIN_CLOCK == 64000000)                 /* 64MHz下对应BGR */
#define LIN_BAUDRATE_9600_BGR           6666 
#define LIN_BAUDRATE_19200_BGR          3333                            
#define LIN_SYN_BAUDRATE_MAX_BGR        159     /* 400K  */
#else                                          /* 8MHz下对应BGR  */
#define LIN_BAUDRATE_9600_BGR           832 
#define LIN_BAUDRATE_19200_BGR          416                             
#define LIN_SYN_BAUDRATE_MAX_BGR        19     /* 400K */
#endif /* End (MCU_BUS_FREQ == 16000000) */


uint32_t    BitLength = LIN_BAUDRATE_19200_BGR; /* 位速率对应的BGR */

#endif

/* 帧格式 [最大11字节] */
/* data区内容：受保护ID-1字节; 数据段-0-8字节；校验和-1字节 */
typedef struct s_lin_frame_type
{
    uint8_t tx_len;/* 发送长度 */
    uint8_t rx_len;/* 接收长度 */
    uint8_t data[FRAME_DATA_MAX_LEN];
} lin_frame_data_type;

/* 任务处理：等待接收并处理 */
typedef struct s_lin_task_type
{
    uint8_t frame_state;  /* 帧处理状态机，参照 FRAME_STATE_IDLE 定义 */

    /* 帧超时计数，接收到同步段后置起，按照8字节数据计算超时 */
    /* 公式：round((1.4x(10+8x10)xTbit)/Tbase_period) + 3，8为接收字节数，Tbit为单个bit用时，Tbase_period为定时器周期，单位都是us */
    uint16_t frame_timeout_cnt;
    /* 响应超时计数，接收到需要响应的ID后置起，按照实际接收长度计算超时 */
    /* 公式：round((1.4x(10+Nx10)xTbit)/Tbase_period) + 3，N为接收字节数，Tbit为单个bit用时，Tbase_period为定时器周期，单位都是us */
    uint16_t res_frame_timeout_cnt;
    /* Lin总线IDLE超时计数，减为0进入SLEEP状态 */
    uint16_t idle_timeout_cnt;
    lin_frame_data_type lin_frame;/* 帧内容 */
} lin_slave_task_type;

lin_slave_task_type lin_slave_task;

uint8_t LIN_SLAVE_CTRL_REQUEST_Data[8];
uint8_t LIN_SLAVE_CTRL_RESPONSE_Data[8] = {0x12, 0x34, 0x56, 0x78};
uint8_t LIN_SLAVE_CMD2_Data[8] = {0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88};
uint8_t LIN_SLAVE_BOOT_REQUEST_Data[8];
uint8_t LIN_SLAVE_BOOT_RESPONSE_Data[8];

lin_slave_cmd_packet_t slave_scheduleTable[] =
{
    /* 用户命令码 */
    {LIN_SLAVE_CTRL_REQUEST, RECEIVE,  LIN_SLAVE_CTRL_REQUEST_LENGTH, LIN_SLAVE_CTRL_REQUEST_Data},
    {LIN_SLAVE_CTRL_RESPONSE, TRANSMIT, LIN_SLAVE_CTRL_RESPONSE_LENGTH, LIN_SLAVE_CTRL_RESPONSE_Data},
    {LIN_SLAVE_CMD2, TRANSMIT, LIN_SLAVE_CMD2_LENGTH, LIN_SLAVE_CMD2_Data},
    {LIN_SLAVE_BOOT_REQUEST, RECEIVE,  LIN_SLAVE_BOOT_REQUEST_LENGTH, LIN_SLAVE_BOOT_REQUEST_Data},
    {LIN_SLAVE_BOOT_RESPONSE, TRANSMIT, LIN_SLAVE_BOOT_RESPONSE_LENGTH, LIN_SLAVE_BOOT_RESPONSE_Data},   
};

/**
  * @brief  接收超时定时器初始化
  * @param  void
  * @retval void
  */
void lin_timeout_init(void)
{
    /* 使能TAU时钟 */
    FL_CMU_EnableGroup4BusClock(FL_CMU_GROUP4_BUSCLK_TAU1);

    /* 配置TAU13普通计数模式 */
    FL_TAU_SetOperationMode(TAU13, FL_TAU_OPERATION_MODE_NORMAL);
    FL_TAU_WritePrescaler(TAU13, 0); 
    FL_TAU_WriteAutoReload(TAU13, TIMER_1US_LOAD * TIME_BASE_PERIOD - 1);    
    
    /* NVIC中断优先级 */
    NVIC_DisableIRQ(TAU_PGL_IRQn);
    NVIC_SetPriority(TAU_PGL_IRQn, TAU_PGL_IRQ_PRI);
    NVIC_EnableIRQ(TAU_PGL_IRQn);
    
    FL_TAU_EnableIT_Update(TAU13);
    FL_TAU_Enable(TAU13);  /* 使能定时器 */
}


/**
  * @brief  lin idle（Master）
  * @param  void
  * @retval void
  */
void lin_master_goto_idle(void)
{
    lin_schedule_data_type *plin_shcedule_data;
    
    plin_shcedule_data = &lin_master_task.lin_schedule.lin_schedule_data[lin_master_task.frame_index];
    
    plin_shcedule_data->rx_len = 0;
    
    lin_master_task.frame_state = FRAME_STATE_IDLE;
    
    MasterUARTxOp.RxLen = 0;
    MasterUARTxOp.TxLen = 0;
}

/**
  * @brief  lin idle
  * @param  void
  * @retval void
  */
void lin_slave_goto_idle(void)
{
    START_CRITICAL();

    ENTER_CRITICAL();

    lin_slave_task.lin_frame.tx_len = 0;
    lin_slave_task.lin_frame.rx_len = 0;
    lin_slave_task.frame_state = FRAME_STATE_IDLE;
    lin_slave_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
    
    SlaveUARTxOp.RxLen = 0;
    SlaveUARTxOp.TxLen = 0;
    
    #if (AUTOBAUD == 1)
    /* 关闭同步段中断 */
    LIN_SLAVE_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);     /* 清同步段标志 */
    LIN_SLAVE_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
    
    /* 切换波特率为初始最高波特率 */
    LIN_SLAVE_UART->BGR = BitLength - 1;
    #else
    FL_UART_EnableRX(LIN_SLAVE_UART);                /* 非自动波特率打开接收 */
    #endif

    EXIT_CRITICAL();
}


/**
  * @brief  lin idle state初始化(slave)
  * @param  void
  * @retval void
  */
__STATIC_INLINE void lin_slave_init_idle_state(void)
{        
    lin_slave_task.lin_frame.tx_len = 0;
    lin_slave_task.lin_frame.rx_len = 0;
    lin_slave_task.frame_state = FRAME_STATE_IDLE;
    lin_slave_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
    
    SlaveUARTxOp.RxLen = 0;
    SlaveUARTxOp.TxLen = 0;
}


/**
  * @brief  定时，用于判断字节间超时
  * @note   12位波特率，如1200-10ms 19.2KHz-625us
  * @param  void
  * @retval void
  */
void TAU13_IRQHandler(void)
{
    if (FL_TAU_IsActiveFlag_Update(TAU13))
    {
        FL_TAU_ClearFlag_Update(TAU13);
        
        /* Master */
        switch (lin_master_task.frame_state)
        {
            case FRAME_STATE_IDLE:
              /* 之后增加休眠超时判定 */
              break;
            case FRAME_STATE_SEND_SYN:
              break;
            case FRAME_STATE_SEND:
              if (0 == lin_master_task.frame_timeout_cnt)
              {
                  lin_master_goto_idle();
              }
              else
              {
                  lin_master_task.frame_timeout_cnt--;
              }
              break;
            case FRAME_STATE_RECV_DATA:
              if (0 == lin_master_task.res_frame_timeout_cnt)
              {
                
                  if (MasterUARTxOp.RxLen)
                  {
                      MasterUARTxOp.RxLen = 0;
                  }
                  lin_master_goto_idle();
              }
              else
              {
                  lin_master_task.res_frame_timeout_cnt--;
              }            
              break;
            default:
              break;
        }
        
        /* Slave */
        switch (lin_slave_task.frame_state)
        {
            case FRAME_STATE_IDLE:
                if (0 == lin_slave_task.idle_timeout_cnt)
                {
                    lin_slave_task.frame_state = FRAME_STATE_SLEEP;
                    lin_slave_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
                }
                else
                {
                    lin_slave_task.idle_timeout_cnt--;
                }
                break;
            case FRAME_STATE_RECV_SYN:
            case FRAME_STATE_RECV_PID:
            case FRAME_STATE_SEND:
                if (0 == lin_slave_task.frame_timeout_cnt)
                {
                    lin_slave_goto_idle();
                }
                else
                {
                    lin_slave_task.frame_timeout_cnt--;
                }
                break;
            case FRAME_STATE_RECV_DATA:
                if (0 == lin_slave_task.res_frame_timeout_cnt)
                {
                    
                    if (SlaveUARTxOp.RxLen > 1)
                    {
                        SlaveUARTxOp.RxLen = 0;
                    }
                    lin_slave_goto_idle();
                }
                else
                {
                    lin_slave_task.res_frame_timeout_cnt--;
                }
                break;
        }
        
    }
}

/**
  * @brief  LIN主机发送同步段(master)
  * @param  plin_shcedule_data: 进度表指针
  * @retval void
  */
void lin_master_send_syn(lin_schedule_data_type *plin_shcedule_data)
{
    plin_shcedule_data->frame_left_slot = plin_shcedule_data->frame_slot;
    
    lin_master_task.frame_state = FRAME_STATE_SEND_SYN;
    
    LIN_MASTER_UART->LINCR |= UART_LINCR_TX_BF;
}

/**
  * @brief  LPTIM中断函数
  * @param  void
  * @retval void
  */
void LPTIM32_IRQHandler(void)
{
    static uint32_t count_master = 0, flag_master = 0;
    uint8_t i;
   
    /***************** 更新帧间隙 ***************/
    lin_schedule_data_type *plin_master_shcedule_data;
  
    plin_master_shcedule_data = &lin_master_task.lin_schedule.lin_schedule_data[lin_master_task.frame_index];
    
    if (LPTIM32->ISR & LPTIM32_ISR_OVIF_Msk)
    {     
        /***************** Master ***************/
        if (plin_master_shcedule_data->frame_left_slot)
        {
            plin_master_shcedule_data->frame_left_slot--;
        }

        if (flag_master == 0)
        {
            flag_master = 1;

            lin_master_task.frame_index = 0;

            plin_master_shcedule_data = &lin_master_task.lin_schedule.lin_schedule_data[lin_master_task.frame_index];
            lin_master_send_syn(plin_master_shcedule_data);
        }

        /* 寻找切换时刻 */
        for (i = 0; i < MASTER_TABLE_SIZE; i++)
        {
            if(count_master == schedule_master_table_slot[i])
            {
                lin_master_task.frame_index++;
                if(lin_master_task.frame_index >= MASTER_TABLE_SIZE)
                {
                    lin_master_task.frame_index = 0;
                    count_master = 0;
                }
                plin_master_shcedule_data = &lin_master_task.lin_schedule.lin_schedule_data[lin_master_task.frame_index];
                lin_master_send_syn(plin_master_shcedule_data);
              
                break;
            }
        }

        count_master++;        
        
        LPTIM32->ISR = LPTIM32_ISR_OVIF_Msk;
    }
}

/**
  * @brief  LIN-UART中断函数
  * @param  void
  * @retval void
  */
void LIN_MASTER_UART_IRQHandler(void)
{
    uint8_t tmp08;
    uint32_t timeout;
    lin_schedule_data_type *plin_shcedule_data;
    
    uint32_t isr_flag = 0;
    static uint8_t init_flag = 0;    
    
    isr_flag = LIN_MASTER_UART->ISR;    
    
    plin_shcedule_data = &lin_master_task.lin_schedule.lin_schedule_data[lin_master_task.frame_index];    
    
    if ((isr_flag & UART_ISR_LINWKDF_Msk) &&
        (FL_SET == FL_UART_IsEnabledIT_LINWakeUp(LIN_MASTER_UART))) /* 收到唤醒信号 */
    {
        #if (SLEEP_SUPPORT == 1)
        if (g_master_awake_init == 1)
        {
            g_master_awake_init = 0;

            /* lin唤醒后配置 */
            lin_master_awake_init(1);
        }
        #endif
        
        FL_UART_ClearFlag_LINDetectWakeUpSignal(LIN_MASTER_UART);   /* 清除唤醒信号检测标志 */
        FL_UART_DisableIT_LINWakeUp(LIN_MASTER_UART);               /* 关闭唤醒中断 */
        FL_UART_DisableLINWkupDetect(LIN_MASTER_UART);              /* 关闭唤醒检测 */
        
        if (init_flag == 0)
        {
            lin_master_goto_idle();
            init_flag = 1;
        }

        LIN_MASTER_UART->ISR = LIN_MASTER_UART->ISR;
        return;
    }    
    
    if ((isr_flag & UART_ISR_LINBF_Msk) &&
        (LIN_MASTER_UART->IER & UART_IER_LINBIE_Msk)) /* 收到break段 */
    {    
        /* Clear flag */
        LIN_MASTER_UART->ISR = UART_ISR_LINBF_Msk;    /* 清break段接收完成标志 */
        
        if (lin_master_task.frame_state != FRAME_STATE_SEND_SYN)
        {
            /* 当前状态不正确，直接结束任务 */
            lin_master_goto_idle();            
        }
        else
        {
            /* 第一时间发送第一个字节 0x55 */
            LIN_MASTER_UART->TXBUF = plin_shcedule_data->data[0];
            
            /* 实时更新buffer */                
            memcpy(&plin_shcedule_data->data[2], master_scheduleTable[lin_master_task.frame_index].data, master_scheduleTable[lin_master_task.frame_index].length);
            #if (CHECKSUM_TYPE == CHECKSUM_TYPE_ENHANCED)
                plin_shcedule_data->data[master_scheduleTable[lin_master_task.frame_index].length + 2] = calculateChecksum(&plin_shcedule_data->data[1], master_scheduleTable[lin_master_task.frame_index].length + 1);
            #else
                plin_shcedule_data->data[master_scheduleTable[lin_master_task.frame_index].length + 2] = calculateChecksum(&plin_shcedule_data->data[2], master_scheduleTable[lin_master_task.frame_index].length);
            #endif
                            
            /* 设置发送长度和数据 */
            MasterUARTxOp.TxOpc = 1;
            MasterUARTxOp.TxLen = plin_shcedule_data->tx_len;
            MasterUARTxOp.TxBuf = plin_shcedule_data->data;
            
            /* 主发送数据阶段 */
            lin_master_task.frame_state = FRAME_STATE_SEND;
            
            /* 收到间隔段之后置起超时，等待在超时内处理后续流程 */
            lin_master_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 8);
            lin_master_task.res_frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, master_scheduleTable[lin_master_task.frame_index].length + 1);  
        }

        return;
    }
   
    /* 接收错误中断处理 需要注意帧类型错误会同时有RXBF标志，用于判定同步段 */
    if (LIN_MASTER_UART->ISR & UART_ISR_FERR_Msk)
    {
        tmp08 = LIN_MASTER_UART->RXBUF;
        lin_master_goto_idle();
        LIN_MASTER_UART->ISR = UART_ISR_FERR_Msk;
    }
    /* 接收中断处理 */
    else if (LIN_MASTER_UART->ISR & UART_ISR_RXBF_Msk)
    {
        switch (lin_master_task.frame_state)
        {
            /* 接收数据阶段，直到接收完成才处理 */
            case FRAME_STATE_RECV_DATA:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_MASTER_UART->RXBUF;
                
                MasterUARTxOp.RxBuf[MasterUARTxOp.RxLen] = tmp08;
                MasterUARTxOp.RxLen++;
            
                if (MasterUARTxOp.RxLen == master_scheduleTable[lin_master_task.frame_index].length + 1)
                {
                    /* 接收完整数据 */
                    memcpy(&plin_shcedule_data->data[2], MasterUARTxOp.RxBuf, MasterUARTxOp.RxLen);
    
                    #if (CHECKSUM_TYPE == CHECKSUM_TYPE_ENHANCED)
                        if (calculateChecksum(&plin_shcedule_data->data[1], MasterUARTxOp.RxLen) == MasterUARTxOp.RxBuf[MasterUARTxOp.RxLen - 1])
                        {
                            plin_shcedule_data->rx_len = MasterUARTxOp.RxLen;
                            memcpy(master_scheduleTable[lin_master_task.frame_index].data, MasterUARTxOp.RxBuf, master_scheduleTable[lin_master_task.frame_index].length);
                        }
                        else
                        {
                            plin_shcedule_data->rx_len = 0;                    
                        }
                    #else
                        if (calculateChecksum(&plin_shcedule_data->data[2], MasterUARTxOp.RxLen - 1) == MasterUARTxOp.RxBuf[MasterUARTxOp.RxLen - 1])
                        {
                            plin_shcedule_data->rx_len = MasterUARTxOp.RxLen;
                            memcpy(master_scheduleTable[lin_master_task.frame_index].data, MasterUARTxOp.RxBuf, master_scheduleTable[lin_master_task.frame_index].length);
                        }
                        else
                        {
                            plin_shcedule_data->rx_len = 0;
                        }
                    #endif
                    lin_master_goto_idle();
                }
                
                break;
            case FRAME_STATE_SEND:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_MASTER_UART->RXBUF;
            
                /* 发送指定长度的数据 */
                if (MasterUARTxOp.TxOpc < MasterUARTxOp.TxLen)
                {
                    LIN_MASTER_UART->TXBUF = MasterUARTxOp.TxBuf[MasterUARTxOp.TxOpc]; 
                    MasterUARTxOp.TxOpc++;
                }
                else
                {
                    /* 发送完毕 */
                    if (plin_shcedule_data->frame_type == FRAME_TYPE_NO_RESPONSE)
                    {
                        /* 帧类型为无需回应 */                
                        lin_master_goto_idle();
                    }
                    else
                    {
                        /* 需等待回应 */
                        lin_master_task.frame_state = FRAME_STATE_RECV_DATA;
                        
                        /* 开启应答间隔判断 */
                        /* 加入余量控制的超时，单位ms */
                        if (plin_shcedule_data->timeout == 0)
                        {
                            timeout =  plin_shcedule_data->frame_left_slot;
                        }
                        else
                        {
                            timeout = Min(plin_shcedule_data->timeout, plin_shcedule_data->frame_left_slot);
                        }
                        
                        timeout = timeout * 1000 / TIME_BASE_PERIOD;
                        
                        lin_master_task.res_frame_timeout_cnt = Min(timeout, lin_master_task.res_frame_timeout_cnt);
                    }
                }
                break;
            default:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_MASTER_UART->RXBUF;
            
                break;
        }
    }
    
}

/**
  * @brief  UART-LIN中断
  * @param  void
  * @retval void
  */
void LIN_SLAVE_UART_IRQHandler(void)
{
    uint8_t tmp08;
    static uint8_t i;
    
    #if (AUTOBAUD == 1)
    uint32_t isr_flag = 0;
    uint32_t tmp_32;
    static uint8_t init_flag = 0;    
    
    isr_flag = LIN_SLAVE_UART->ISR;
    
    /******************************
    *** 0. WAKEUP DETECTED
    *******************************/ 
    if ((isr_flag & UART_ISR_LINWKDF_Msk) &&
        (FL_SET == FL_UART_IsEnabledIT_LINWakeUp(LIN_SLAVE_UART)))/* 收到唤醒信号 */
    {
        #if (SLEEP_SUPPORT == 1)
        if (g_slave_awake_init == 1)
        {
            g_slave_awake_init = 0;

            /* lin唤醒后配置 */
            lin_slave_awake_init(1);
        }
        #endif

        FL_UART_ClearFlag_LINDetectWakeUpSignal(LIN_SLAVE_UART);  /* 清除唤醒信号检测标志 */
        FL_UART_DisableIT_LINWakeUp(LIN_SLAVE_UART);              /* 关闭唤醒中断 */
        FL_UART_DisableLINWkupDetect(LIN_SLAVE_UART);             /* 关闭唤醒检测 */
        
        if (init_flag == 0)
        {
            lin_slave_goto_idle();
            init_flag = 1;
        }

        LIN_SLAVE_UART->ISR = LIN_SLAVE_UART->ISR;
        return;
    }

    if ((isr_flag & UART_ISR_LINBF_Msk) &&
        (LIN_SLAVE_UART->IER & UART_IER_LINBIE_Msk)) /* 收到break段 */
    {
        /* Clear flag */
        LIN_SLAVE_UART->ISR = UART_ISR_LINBF_Msk;    /* 清break段接收完成标志 */
        {
            lin_slave_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(BitLength, 8);//收到break后超时切换为较大值
        
            LIN_SLAVE_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);//清同步段标志和同步段错误标志
            LIN_SLAVE_UART->IER |= UART_IER_LINSYNCIE_Msk;//开同步段中断，等待接收同步段0x55
                
            lin_slave_task.frame_state = FRAME_STATE_RECV_SYN;  
                
            LIN_SLAVE_UART->ISR = LIN_SLAVE_UART->ISR;
        }
        return;
    }
    
    /******************************
    *** 1.1 SYN DETECTED
    *******************************/
    if ((isr_flag & UART_ISR_LINSYNCF_Msk) &&
        (LIN_SLAVE_UART->IER & UART_IER_LINSYNCIE_Msk))  /* 收到同步段 */
    { 
        if (LIN_SLAVE_UART->ISR & UART_ISR_SER_Msk)
        {
            LIN_SLAVE_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);
            lin_slave_goto_idle();
            LIN_SLAVE_UART->ISR = LIN_SLAVE_UART->ISR;
        }
        else
        {
            LIN_SLAVE_UART->ISR = UART_ISR_LINSYNCF_Msk;     /* 清同步段标志 */
            LIN_SLAVE_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
            
            /* 计算分频系数 */
            tmp_32 = LIN_SLAVE_UART->LINBSR;  /* 获取同步段计数时钟个数 */
            tmp_32 = (tmp_32 >> 3);
            {
                BitLength = tmp_32;

                lin_slave_task.lin_frame.rx_len = 0;
                lin_slave_task.frame_state = FRAME_STATE_RECV_PID;
                SlaveUARTxOp.RxLen = 0;
            }
        }
        LIN_SLAVE_UART->ISR = LIN_SLAVE_UART->ISR;
        return;
    }
    #else
    #if (SLEEP_SUPPORT == 1)
     if ((LIN_SLAVE_UART->ISR & UART_ISR_LINWKDF_Msk) &&
        (FL_SET == FL_UART_IsEnabledIT_LINWakeUp(LIN_SLAVE_UART))) /* 收到唤醒信号 */
    {   
        if (g_slave_awake_init == 1)
        {
            g_slave_awake_init = 2;

            FL_CMU_DisableOperationClock(FL_CMU_OPCLK_LIN);  /* 关闭LIN工作时钟LSCSK=32768，用于唤醒信号检测 */

            LIN_SLAVE_UART->MCR = 0;         /* 配置为普通UART */

            /* lin唤醒后配置 */
            lin_slave_awake_init(1);
        }
        
        FL_UART_ClearFlag_LINDetectWakeUpSignal(LIN_SLAVE_UART);    /* 清除唤醒信号检测标志 */
        FL_UART_DisableIT_LINWakeUp(LIN_SLAVE_UART);                /* 关闭唤醒中断 */
        FL_UART_DisableLINWkupDetect(LIN_SLAVE_UART);               /* 关闭唤醒检测 */
       
        LIN_SLAVE_UART->ISR = LIN_SLAVE_UART->ISR;
        return;
    }
    #endif
    #endif
    
    /* 接收错误中断处理 需要注意帧类型错误会同时有RXBF标志，用于判定同步段 */
    if (LIN_SLAVE_UART->ISR & UART_ISR_FERR_Msk)
    {
        tmp08 = LIN_SLAVE_UART->RXBUF;
        LIN_SLAVE_UART->ISR = UART_ISR_FERR_Msk;
        
        #if (AUTOBAUD == 1)
        lin_slave_goto_idle();
        #else
        if (tmp08 == LIN_BREAK_BYTE)
        {
            /* 检测到间隔段 */
            lin_slave_task.frame_state = FRAME_STATE_RECV_SYN;  
            
            /* 收到间隔段之后置起超时，等待在超时内收到同步字节0x55 */
            lin_slave_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 8);
        }
        else
        {
            /* 帧错误 */
            
            lin_slave_goto_idle();
        }  
        #endif
    }
    /* 接收中断处理 */
    else if (LIN_SLAVE_UART->ISR & UART_ISR_RXBF_Msk)
    {
        switch (lin_slave_task.frame_state)
        {
            /* 固定波特率接收同步阶段 */
            case FRAME_STATE_RECV_SYN:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_SLAVE_UART->RXBUF;
                SlaveUARTxOp.RxLen = 0;
            
                #if (SLEEP_SUPPORT == 1)
                if (g_slave_awake_init == 2)
                {
                    /* 重置超时部分 */
                    g_slave_awake_init = 0;
                    lin_slave_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 8);
                }
                #endif           
            
                if (tmp08 == LIN_SYN_BYTE)
                {
                    /* 间隔段之后的第一个字节是0x55 */
                    lin_slave_task.frame_state = FRAME_STATE_RECV_PID;
                }
                else
                {
                    lin_slave_goto_idle();
                }
                break;
            /* 接收PID阶段，判断PID是否有效如有效判定后续行为是发送还是接收 */
            case FRAME_STATE_RECV_PID:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_SLAVE_UART->RXBUF;
            
                SlaveUARTxOp.RxBuf[SlaveUARTxOp.RxLen] = tmp08;
                SlaveUARTxOp.RxLen++;
            
            
                for (i = 0; i < SLAVE_TABLE_SIZE; i++)
                {
                    if (slave_scheduleTable[i].pid == tmp08)
                    {
                        break;
                    }
                }
                
                
                if (i == SLAVE_TABLE_SIZE)
                {
                    /* 无对应命令，不做处理 */
                    lin_slave_goto_idle();

                    return;
                }
                else
                {
                    if (slave_scheduleTable[i].type == RECEIVE)
                    {
                        /* 主机发送命令，更新待接收长度，转接收数据阶段 */
                        lin_slave_task.lin_frame.rx_len = slave_scheduleTable[i].length + 1;
                        lin_slave_task.frame_state = FRAME_STATE_RECV_DATA;
                        
                        /* 更新响应超时 */
                        #if (AUTOBAUD == 1)
                        lin_slave_task.res_frame_timeout_cnt = lin_calc_max_res_timeout_cnt(BitLength, slave_scheduleTable[i].length);
                        #else
                        lin_slave_task.res_frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, slave_scheduleTable[i].length);
                        #endif
                    }
                    else
                    {
                        if ((0x0 == slave_scheduleTable[i].FullFlg) &&
                            (LIN_SLAVE_BOOT_RESPONSE == slave_scheduleTable[i].cmd))
                        {
                            /* 无数据发送 */
                            lin_slave_goto_idle();
                            return;
                        }
                        else if (LIN_SLAVE_CTRL_RESPONSE == slave_scheduleTable[i].cmd)
                        {
                            static ctrl_data_type temp_ctrl_data;
                            
                            get_current_ctrl_status(&temp_ctrl_data, 5);/* 有响应时间限制，获取历史数据快速响应 */
                            memcpy(slave_scheduleTable[i].data, temp_ctrl_data.data, 8); 
                        }
                        
                        /* 主机接收命令，更新待发送长度，转发送数据阶段 */
                        /* 第一时间发送数据，保证负载率 */
                        LIN_SLAVE_UART->TXBUF = slave_scheduleTable[i].data[0];
                        
                        #if (CHECKSUM_TYPE == CHECKSUM_TYPE_ENHANCED)
                            lin_slave_task.lin_frame.data[0] = PID;
                            memcpy(&lin_slave_task.lin_frame.data[1], slave_scheduleTable[i].data, slave_scheduleTable[i].length);
                            lin_slave_task.lin_frame.data[slave_scheduleTable[i].length + 1] = calculateChecksum(lin_slave_task.lin_frame.data, slave_scheduleTable[i].length + 1);

                            lin_slave_task.lin_frame.tx_len = slave_scheduleTable[i].length + 1;
                            SlaveUARTxOp.TxLen = lin_slave_task.lin_frame.tx_len;
                            SlaveUARTxOp.TxOpc = 1;
                            SlaveUARTxOp.TxBuf = &lin_slave_task.lin_frame.data[1];
                        #else
                            memcpy(lin_slave_task.lin_frame.data, slave_scheduleTable[i].data, slave_scheduleTable[i].length);
                            lin_slave_task.lin_frame.data[slave_scheduleTable[i].length] = calculateChecksum(lin_slave_task.lin_frame.data, slave_scheduleTable[i].length);

                            lin_slave_task.lin_frame.tx_len = slave_scheduleTable[i].length + 1;
                            SlaveUARTxOp.TxLen = lin_slave_task.lin_frame.tx_len;
                            SlaveUARTxOp.TxOpc = 1;
                            SlaveUARTxOp.TxBuf = lin_slave_task.lin_frame.data;
                        #endif
                        lin_slave_task.lin_frame.rx_len = 0;
                        lin_slave_task.frame_state = FRAME_STATE_SEND;                        
                    }
                }
                
                break;
            /* 接收数据阶段，直到接收完成才处理 */
            case FRAME_STATE_RECV_DATA:
                if (SlaveUARTxOp.RxLen >= lin_slave_task.lin_frame.rx_len)
                {
                    /* 最后一个字节，第一时间开接收 */
                    #if (AUTOBAUD == 1)
                    /* 关闭同步段中断 */
                    LIN_SLAVE_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);     /* 清同步段标志 */
                    LIN_SLAVE_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
                    
                    /* 切换波特率为初始最高波特率 */
                    LIN_SLAVE_UART->BGR = BitLength - 1;

                    #else
                    LIN_SLAVE_UART->CSR |= UART_CSR_RXEN_Msk;
                    #endif
                    
                }
                
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_SLAVE_UART->RXBUF;
                
                SlaveUARTxOp.RxBuf[SlaveUARTxOp.RxLen] = tmp08;
                SlaveUARTxOp.RxLen++;
            
                if (SlaveUARTxOp.RxLen >= lin_slave_task.lin_frame.rx_len + 1)
                {
                    /* 主机主动发送，解析数据 */
                    #if (CHECKSUM_TYPE == CHECKSUM_TYPE_ENHANCED)                        
                        if (calculateChecksum(SlaveUARTxOp.RxBuf, SlaveUARTxOp.RxLen - 1) == SlaveUARTxOp.RxBuf[SlaveUARTxOp.RxLen - 1])
                        {
                            
                            /* 通过校验 */
                            if (slave_scheduleTable[i].length != (SlaveUARTxOp.RxLen - 2))
                            {
                                lin_slave_init_idle_state();

                                return;
                            }
                            else
                            {
                                memcpy(slave_scheduleTable[i].data, &SlaveUARTxOp.RxBuf[1], slave_scheduleTable[i].length);
                                if (LIN_SLAVE_BOOT_REQUEST == slave_scheduleTable[i].cmd)
                                {
                                    slave_scheduleTable[i].FullFlg = 0x1U;
                                }
                                else if (LIN_SLAVE_CTRL_REQUEST == slave_scheduleTable[i].cmd)
                                {
                                    g_app_struct.message_src = MESSAGE_FROM_LIN;
                                    memcpy((void *)g_app_struct.ctrl_data.data, slave_scheduleTable[i].data, 8);
                                    g_app_struct.update_flag = 1;
                                }
                            }
                        }
                    #else
                        if (calculateChecksum(&SlaveUARTxOp.RxBuf[1], SlaveUARTxOp.RxLen - 2) == SlaveUARTxOp.RxBuf[SlaveUARTxOp.RxLen - 1])
                        {
                            if (slave_scheduleTable[i].length != (SlaveUARTxOp.RxLen - 2))
                            {
                                lin_slave_init_idle_state();

                                return;
                            }
                            else
                            {
                                memcpy(slave_scheduleTable[i].data, &SlaveUARTxOp.RxBuf[1], slave_scheduleTable[i].length);
                                if (LIN_SLAVE_BOOT_REQUEST == slave_scheduleTable[i].cmd)
                                {
                                    slave_scheduleTable[i].FullFlg = 0x1U;
                                }
                                else if (LIN_SLAVE_CTRL_REQUEST == slave_scheduleTable[i].cmd)
                                {
                                    g_app_struct.message_src = MESSAGE_FROM_LIN;
                                    memcpy((void *)g_app_struct.ctrl_data.data, slave_scheduleTable[i].data, 8);
                                    g_app_struct.update_flag = 1;
                                }
                            }
                        }
                    #endif
                        lin_slave_init_idle_state();
                }
                break;
            case FRAME_STATE_SEND:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_SLAVE_UART->RXBUF;

                if (SlaveUARTxOp.TxOpc < SlaveUARTxOp.TxLen)
                {
                    if (tmp08 != SlaveUARTxOp.TxBuf[SlaveUARTxOp.TxOpc - 1])
                    {   
                        /* bit错误 */
                        #if (AUTOBAUD == 1)
                        /* 关闭同步段中断 */
                        LIN_SLAVE_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);     /* 清同步段标志 */
                        LIN_SLAVE_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
                        
                        /* 切换波特率为初始最高波特率 */
                        LIN_SLAVE_UART->BGR = BitLength - 1;
                        #else
                        LIN_SLAVE_UART->CSR |= UART_CSR_RXEN_Msk;
                        #endif
                                      
                        
                        lin_slave_init_idle_state();
                    }
                    else
                    {
                        LIN_SLAVE_UART->TXBUF = SlaveUARTxOp.TxBuf[SlaveUARTxOp.TxOpc];  /* 发送一个数据 */
                        SlaveUARTxOp.TxOpc++;
                    }
                }
                else
                {
                    /* 发送完毕，快速切换至接收 */
                    #if (AUTOBAUD == 1)
                    /* 关闭同步段中断 */
                    LIN_SLAVE_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);     /* 清同步段标志 */
                    LIN_SLAVE_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
                    
                    /* 切换波特率为初始最高波特率 */
                    LIN_SLAVE_UART->BGR = BitLength - 1;
                    #else
                    LIN_SLAVE_UART->CSR |= UART_CSR_RXEN_Msk;
                    #endif
                    
                    
                    lin_slave_init_idle_state();
                    
                    if (LIN_SLAVE_BOOT_RESPONSE == slave_scheduleTable[i].cmd)
                    {
                        slave_scheduleTable[i].FullFlg = 0x0;
                    }
                }
                break;
            default:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_SLAVE_UART->RXBUF;
            
                break;
        }
    }    
    
    /* 清除可能的错误中断标志 */
    if (LIN_SLAVE_UART->ISR & UART_ISR_PERR_Msk)
    {
        LIN_SLAVE_UART->ISR = UART_ISR_PERR_Msk;
    }
    if (LIN_SLAVE_UART->ISR & UART_ISR_OERR_Msk)
    {
        LIN_SLAVE_UART->ISR = UART_ISR_OERR_Msk;
    }
}

/**
  * @brief  lin UART初始化
  * @param  void
  * @retval void
  */
void lin_master_uart_init(UART_Type * UARTx)
{
    /* 初始化uart配置 */
    Uartx_Init(UARTx);
    UARTx->MCR |= UART_MCR_MODE_Msk;    /* 配置模块为LIN模式、默认为主机模式 */
    UARTx->MCR &= ~UART_MCR_LINMD_Msk;  /* 主机模式 */
    
    FL_CMU_EnableOperationClock(FL_CMU_OPCLK_LIN);  /* 使能LIN工作时钟LSCSK=32768，用于唤醒信号检测 */

    FL_UART_EnableIT_LINBreakField(UARTx);       /* 开break中断使能，等待接收break段 */
    
    /* 接收错误中断使能，用于同步判断 */
    FL_UART_EnableIT_RXError(UARTx);
    FL_UART_EnableIT_RXBuffFull(UARTx);
    
    FL_UART_EnableTX(UARTx);
    FL_UART_EnableRX(UARTx);
}

/**
  * @brief  lin UART初始化
  * @param  void
  * @retval void
  */
void lin_slave_uart_init(UART_Type * UARTx)
{
    /* 初始化uart配置 */
    Uartx_Init(UARTx);

    /* 中断使能 */
    #if (AUTOBAUD == 1)
    UARTx->MCR |= 0x1U;   /* 配置模块为LIN模式、默认为主机模式 */
    UARTx->MCR |= 0x2U;   /* 从机模式 */
    
    FL_CMU_EnableOperationClock(FL_CMU_OPCLK_LIN);      /* 使能LIN工作时钟LSCSK=32768，用于唤醒信号检测 */
    
    /* UART复位后处于Sleep状态，必须等待唤醒信号唤醒才会切换状态 */
    FL_UART_ClearFlag_LINDetectWakeUpSignal(UARTx);  /* 清除唤醒信号检测标志 */
    FL_UART_EnableIT_LINWakeUp(UARTx);               /* 使能唤醒中断 */
    FL_UART_SetLINWkupLength(UARTx, 4);              /* 5*30.5=152.5us  */
    FL_UART_EnableLINWkupDetect(UARTx);              /* 使能唤醒检测 */
    
    FL_UART_EnableIT_LINBreakField(UARTx);           /* 开break中断使能，等待接收break段 */
    #else
    /* 接收错误中断使能，用于同步判断 */
    FL_UART_EnableIT_RXError(UARTx);
    #endif
    /* 接收中断使能 */
    FL_UART_EnableIT_RXBuffFull(UARTx);
    FL_UART_EnableTX(UARTx);
    FL_UART_EnableRX(UARTx);
}

/**
  * @brief  LPTIM32初始化
  * @note   lin任务时基定时器 1ms
  * @param  void
  * @retval void
  */
void LPTIM32_Setup(void)
{
    FL_LPTIM32_InitTypeDef    TimInit;
 
    /*---------------- 定时器时间基准配置 ----------------*/
    TimInit.clockSource          = FL_CMU_LPTIM32_CLK_SOURCE_APBCLK;    /* 工作时钟源选择：APBCLK */
    TimInit.mode                 = FL_LPTIM32_OPERATION_MODE_NORMAL;    /* 工作模式选择：  NORMAL */
    TimInit.prescalerClockSource = FL_LPTIM32_CLK_SOURCE_INTERNAL;      /* 分频时钟源选择：内部时钟 */
    TimInit.prescaler            = FL_LPTIM32_PSC_DIV1;                 /* 预分频选择：    1分频 */
    TimInit.autoReload           = TIMER_1MS_LOAD - 1;                            /* 自动重装载值：  1000 */
    TimInit.onePulseMode         = FL_LPTIM32_ONE_PULSE_MODE_CONTINUOUS;/* 计数模式设置：  连续计数模式 */
    TimInit.triggerEdge          = FL_LPTIM32_ETR_TRIGGER_EDGE_RISING;  /* 触发边沿：      上升沿 */
    TimInit.countEdge            = FL_LPTIM32_ETR_COUNT_EDGE_RISING;    /* 计数边沿：      上升沿 */
    FL_LPTIM32_Init(LPTIM32, &TimInit);
  
    /*-------------------- 中断配置 ----------------------*/
    /* 清除定时器溢出中断标志 */
    FL_LPTIM32_ClearFlag_Update(LPTIM32);
    /* 定时器溢出中断使能 */
    FL_LPTIM32_EnableIT_Update(LPTIM32);

    /* 使能并配置NVIC */
    NVIC_DisableIRQ(LPTIM_BSTIM_IRQn);
    NVIC_SetPriority(LPTIM_BSTIM_IRQn, LPTIM_BSTIM_IRQ_PRI); 
    NVIC_EnableIRQ(LPTIM_BSTIM_IRQn);
    
    /* 使能LPTIM32 */
    FL_LPTIM32_Enable(LPTIM32);    
}

/**
  * @brief  lin master任务初始化
  * @param  void
  * @retval void
  */
void lin_master_task_init(void)
{
    uint8_t index = 0, i;
    lin_schedule_data_type *plin_shcedule_data;
    
    /* 状态机相关初始化 */
    lin_master_task.frame_state = FRAME_STATE_IDLE;
    lin_master_task.frame_index = 0;
    
    /* 进度表初始化 */
    lin_master_task.lin_schedule.frame_num = MASTER_TABLE_SIZE;

    for (i = 0; i < MASTER_TABLE_SIZE; i++)
    {
        plin_shcedule_data = &lin_master_task.lin_schedule.lin_schedule_data[i];
        
        plin_shcedule_data->frame_slot = master_scheduleTable[i].period;
        if (i == 0)
        {
            schedule_master_table_slot[i] = master_scheduleTable[i].period;
        }
        else
        {
            schedule_master_table_slot[i] = schedule_master_table_slot[i - 1] + master_scheduleTable[i].period;
        }
        
        if (master_scheduleTable[i].type == TRANSMIT)
        {
            plin_shcedule_data->frame_type = FRAME_TYPE_NO_RESPONSE;
        }
        else
        {
            plin_shcedule_data->frame_type = FRAME_TYPE_EVENT_TRIGGER;
        }
        plin_shcedule_data->rx_len = 0;
        plin_shcedule_data->timeout = master_scheduleTable[i].timeout;
        
        /* 准备数据区 */
        index = 0;
        plin_shcedule_data->data[index] = 0x55;
        index += 1;
        plin_shcedule_data->data[index] = calculatePID(master_scheduleTable[i].cmd);
        index += 1;
        if (master_scheduleTable[i].type == TRANSMIT)
        {
            memcpy(&plin_shcedule_data->data[index], master_scheduleTable[i].data, master_scheduleTable[i].length);
            index += master_scheduleTable[i].length;
            #if (CHECKSUM_TYPE == CHECKSUM_TYPE_ENHANCED)
                plin_shcedule_data->data[index] = calculateChecksum(&plin_shcedule_data->data[1], master_scheduleTable[i].length + 1);
            #else
                plin_shcedule_data->data[index] = calculateChecksum(&plin_shcedule_data->data[2], master_scheduleTable[i].length);
            #endif
            index += 1;            
        }
        plin_shcedule_data->tx_len = index;
    }
}

/**
  * @brief  lin 任务初始化
  * @param  void
  * @retval void
  */
void lin_slave_task_init(void)
{
    uint8_t i;
    
    /* 状态机相关初始化 */
    lin_slave_task.frame_state = FRAME_STATE_IDLE;
    lin_slave_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;

    /* 帧数据初始化 */
    lin_slave_task.lin_frame.rx_len = 0;
    lin_slave_task.lin_frame.tx_len = 0;
    
    /* 根据cmd计算pid */
    for (i = 0; i < SLAVE_TABLE_SIZE; i++)
    {
        slave_scheduleTable[i].pid = calculatePID(slave_scheduleTable[i].cmd);
    }
}

/**
* @brief  lin 主机初始化(master)
  * @param  void
  * @retval void
  */
void lin_master_init(void)
{
    /* LIN任务初始化 */
    lin_master_task_init();

    /* LIN使能引脚初始化(做预留) */
    lin_master_slp_pin_init();

    /*LIN UART初始化 */
    lin_master_uart_init(LIN_MASTER_UART);
}

#if (AUTOBAUD == 1 )
/**
  * @brief  自适应波特率初始化
  * @param  void
  * @retval void
  */
void AutoBaud_Init(void)
{

}
#endif   /*(AUTOBAUD == 1 )*/

/**
* @brief  LIN从机初始化
  * @param  void
  * @retval void
  */
void lin_slave_init(void)
{
    lin_slave_task_init();
    
    lin_slave_slp_pin_init();
    
    lin_slave_uart_init(LIN_SLAVE_UART);
    
    
    #if (AUTOBAUD == 1)
    AutoBaud_Init();
    #else   
    #endif
}

/**
  * @brief  lin 初始化
  * @param  void
  * @retval void
  */
void lin_init(void)
{
    /* LIN主机初始化 */
    lin_master_init();

    /*LIN从机初始化*/
    lin_slave_init();

    /* 使用定时器用作同步段发送和接收超时判断 */
    lin_timeout_init();

    /* 1ms定时器，LIN Master 主任务处理 */
    LPTIM32_Setup();
}

#if (SLEEP_SUPPORT == 1)
/**
  * @brief  LIN Master 休眠前初始化
  * @param  void
  * @retval void
  */
void lin_master_sleep_init(void)
{
    /* 关闭非唤醒外设 */
    FL_LPTIM32_Disable(LPTIM32);
    
    /* 清除标志 */
    FL_LPTIM32_ClearFlag_Update(LPTIM32);
    NVIC_ClearPendingIRQ(LPTIM_BSTIM_IRQn);    
    
    /* 唤醒源配置 */
    LIN_MASTER_SLP_ENABLE(); /* 收发器休眠 */
    /* Master唤醒配置 */
    FL_UART_ClearFlag_LINDetectWakeUpSignal(LIN_MASTER_UART);    /* 清除唤醒信号检测标志 */
    FL_UART_EnableIT_LINWakeUp(LIN_MASTER_UART);                 /* 使能唤醒中断 */
    FL_UART_SetLINWkupLength(LIN_MASTER_UART, 4);           /* 5*30.5=152.5us */
    FL_UART_EnableLINWkupDetect(LIN_MASTER_UART);          /* 使能唤醒检测 */
    
    LIN_MASTER_UART->ISR = LIN_MASTER_UART->ISR;
    NVIC_ClearPendingIRQ(LIN_MASTER_UART_IRQType);

    /* 关闭非唤醒IO口 */
////    CloseIO(LIN_MASTER_UART_TX_GPIO, LIN_MASTER_UART_TX_PIN);
}
    

/**
  * @brief  LIN Master唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void lin_master_awake_init(uint8_t awake_type)
{   
    if (awake_type == 0)
    {
        g_master_awake_init = 0;
    }
    
    /* 收发器正常 */
    LIN_MASTER_SLP_DISABLE();
    
    /* 打开非唤醒IO */
////    AltFunIO( LIN_MASTER_UART_TX_GPIO, LIN_MASTER_UART_TX_PIN, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, LIN_MASTER_UART_TX_REMAP_FUNCTION, FL_GPIO_DRIVESTRENGTH_X3 );
    
    /* 任务初始化 */
    lin_master_task.frame_state = FRAME_STATE_IDLE;
//    lin_master_task.frame_index = 0;/* 继续任务列表 */
  
    MasterUARTxOp.RxLen = 0;
    MasterUARTxOp.TxLen = 0;

    /* 唤醒源处理 */
    
    /* 非唤醒外设处理 */
    FL_TAU_ClearFlag_Update(TAU13);
    FL_TAU_EnableIT_Update(TAU13);  
    NVIC_EnableIRQ(TAU_PGL_IRQn);
    FL_TAU_Enable(TAU13);
    FL_LPTIM32_Enable(LPTIM32);
}

/**
  * @brief  LIN Master sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t lin_master_is_sleep(void)
{
    if (lin_master_task.frame_state == FRAME_STATE_IDLE)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/**
  * @brief  LIN Slave 休眠前初始化
  * @param  void
  * @retval void
  */
void lin_slave_sleep_init(void)
{
    /* 关闭非唤醒外设 */
    FL_TAU_Disable(TAU13);  /* 禁能定时器 */
    FL_TAU_DisableIT_Update(TAU13);
    FL_TAU_ClearFlag_Update(TAU13);
    Close_And_ClearIRQ(TAU_PGL_IRQn);
    
    /* 唤醒源配置 */
    LIN_SLAVE_SLP_ENABLE(); /* 收发器休眠 */
    /* Slave唤醒配置 */
    #if (AUTOBAUD == 1)
    FL_UART_ClearFlag_LINDetectWakeUpSignal(LIN_SLAVE_UART);    /* 清除唤醒信号检测标志 */
    FL_UART_EnableIT_LINWakeUp(LIN_SLAVE_UART);                 /* 使能唤醒中断 */    
    FL_UART_SetLINWkupLength(LIN_SLAVE_UART, 1);  /* 减小唤醒信号检测时间，以同时检测唤醒和检测 */
    FL_UART_EnableLINWkupDetect(LIN_SLAVE_UART);          /* 使能唤醒检测 */    
    #else
    LIN_SLAVE_UART->MCR |= 0x1U;   /* 配置模块为LIN模式、默认为主机模式 */
    LIN_SLAVE_UART->MCR |= 0x2U;   /* 从机模式 */
    
    FL_CMU_EnableOperationClock(FL_CMU_OPCLK_LIN);          /* 使能LIN工作时钟LSCSK=32768，用于唤醒信号检测 */
    
    /* UART复位后处于Sleep状态，必须等待唤醒信号唤醒才会切换状态 */
    FL_UART_ClearFlag_LINDetectWakeUpSignal(LIN_SLAVE_UART);    /* 清除唤醒信号检测标志 */
    FL_UART_EnableIT_LINWakeUp(LIN_SLAVE_UART);                 /* 使能唤醒中断 */
    FL_UART_SetLINWkupLength(LIN_SLAVE_UART, 4);       /* 5*30.5=152.5us */
    FL_UART_EnableLINWkupDetect(LIN_SLAVE_UART);          /* 使能唤醒检测 */
    #endif

    LIN_SLAVE_UART->ISR = LIN_SLAVE_UART->ISR;
    NVIC_ClearPendingIRQ(LIN_SLAVE_UART_IRQType);

    /* 关闭非唤醒IO口 */
////    CloseIO(LIN_SLAVE_UART_TX_GPIO, LIN_SLAVE_UART_TX_PIN);
}
    

/**
  * @brief  LIN Slave 唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void lin_slave_awake_init(uint8_t awake_type)
{ 
    if (awake_type == 0)
    {
        g_slave_awake_init = 0;
    }
    
    /* 收发器正常 */
    LIN_SLAVE_SLP_DISABLE();
    
    /* 打开非唤醒IO */
////    AltFunIO( LIN_SLAVE_UART_TX_GPIO, LIN_SLAVE_UART_TX_PIN, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, LIN_SLAVE_UART_TX_REMAP_FUNCTION, FL_GPIO_DRIVESTRENGTH_X3 );  
    
    /* 任务初始化 */
    lin_slave_task.lin_frame.tx_len = 0;
    lin_slave_task.lin_frame.rx_len = 0;
    lin_slave_task.frame_state = FRAME_STATE_IDLE;
    lin_slave_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
    
    SlaveUARTxOp.RxLen = 0;
    SlaveUARTxOp.TxLen = 0;

    /* 唤醒源处理 */
    if (awake_type == 1)
    {
        #if (AUTOBAUD == 1)
        #else
        /* 固定波特率唤醒，默认为接收到间隔段，超时改小便于容错 */
        lin_slave_task.frame_state = FRAME_STATE_RECV_SYN;  
        /* 收到间隔段之后置起超时，等待在超时内收到同步字节0x55 */
        lin_slave_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 1);
        #endif        
    }
    
    /* 非唤醒外设处理 */
}

/**
  * @brief  LIN Slave sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t lin_slave_is_sleep(void)
{
    if (lin_slave_task.frame_state == FRAME_STATE_SLEEP)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif /* End #if (SLEEP_SUPPORT == 1) */

int32_t LIN_SLAVE_BOOT_TxMsgFill(uint8_t *data, uint32_t lenth, uint32_t dummy)
{
    uint32_t u32Delay = 0xFFF;
    
    /* 查询前帧是否发送完毕*/
    do
    {
        u32Delay--;
    }while((0x1 == slave_scheduleTable[4].FullFlg) && (0 != u32Delay));
    
    /* 不足填充0xFF */
    memset(slave_scheduleTable[4].data, 0xFF, slave_scheduleTable[4].length);
    if (lenth > slave_scheduleTable[4].length)
    {
        lenth = slave_scheduleTable[4].length;
    }
    memcpy(slave_scheduleTable[4].data, data, lenth);
    /* 置有数据标志 */
    slave_scheduleTable[4].FullFlg = 0x1U;
    return 0;
}
