 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "main.h"
#include "fm33fg0xxa_fl.h"
#include "iwdt.h"
#include "svd.h"
#include "rmu.h"
#include "clock.h"
#include "led.h"
#include "bootloader.h"
#include "sleep.h"
#include "svd_interrupt.h"
#include "i2c.h"
#include "flash.h"
#include "ecc.h"
#include "verify.h"
#include "lptim16.h"

uint32_t main_loop_timer = 0;
volatile app_struct_type g_app_struct;

/**
  * @brief  ECC检错中断不可屏蔽，直接进入NMI中断服务 请保留 
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
    uint32_t RAMDetectionFlag = 0, DataFlashDetectionFlag = 0, CodeFlashDetectionFlag = 0;
    uint32_t WakeUpDebugFlag = 0;
    
    /* RAM ECC 检错  必须保留 */
    RAMDetectionFlag = FL_FLASH_IsActiveFlag_ECCDetectionInRAM(FLASH);
    if (0x01U == RAMDetectionFlag)
    {   
        FL_FLASH_ClearFlag_ECCDetectionInRAM(FLASH);
        RAMEccErrorAdd = FL_FLASH_ReadECCErrorAddressInRAM(FLASH);
        /* 客户安全处理 */
        /* ...... */
    }
    
    /* Data Flash ECC检错  必须保留 */ 
    DataFlashDetectionFlag = FL_FLASH_IsActiveFlag_ECCDetectionInDataFlash(FLASH);
    if (0x01U == DataFlashDetectionFlag)
    {
        FL_FLASH_ClearFlag_ECCDetectionInDataFlash(FLASH);
        DataFlashEccErrorAdd = FL_FLASH_ReadECCErrorAddressInDataFlash(FLASH);
        /* 客户安全处理 */
        /* ...... */
    }
    
    /* Code Flash ECC检错  必须保留 */ 
    CodeFlashDetectionFlag = FL_FLASH_IsActiveFlag_ECCDetectionInCodeFlash(FLASH);
    if (0x01U == CodeFlashDetectionFlag)
    {
        FL_FLASH_ClearFlag_ECCDetectionInCodeFlash(FLASH);
        CodeFlashEccErrorAdd = FL_FLASH_ReadECCErrorAddressInCodeFlash(FLASH);    
        /* 客户安全处理 */
        /* ...... */
    }
    
    /* CPU Debugger唤醒 必须保留 */
    WakeUpDebugFlag = FL_PMU_IsActiveFlag_WakeupDBG(PMU);
    if (0x01U == WakeUpDebugFlag)
    {
        FL_PMU_ClearFlag_WakeupDBG(PMU);
    }
}

/**
  * @brief  HardFault 中断服务函数 请保留 
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{   /* 软件复位MCU，使用内联函数 */
    RMU_Soft_SystemReset();
}

/**
  * @brief  掉电监控，用于执行掉电事件
  * @param  None
  * @retval None
  */
static void PowerDownMonitoring(void)
{
    app_struct_type temp_app_struct;
    uint32_t primask;    
        
    /* 中断全局变量，竞态保护 */
    primask = __get_PRIMASK();
    __disable_irq();
    memcpy((void *)&temp_app_struct, (void *)&g_app_struct, sizeof(app_struct_type));
    __set_PRIMASK(primask);
    
    /* 确认SVD监测结果是否低于阈值 */
    if (true == SVD_Result_Confirmed(SVD_BELOW_THRESHOLD, 1U))
    {   /* 防抖处理 */
        if (true == SVD_Result_Confirmed(SVD_BELOW_THRESHOLD, 2000U/*us*/))
        {
            /* 电压下降到报警阈值，处理掉电事件 */
            /* 用户程序1 */
            #if (EEPROM_SUPPORT == 1)
            WriteE2WithBackup(I2C1, ee_ctrldata_page, (uint8_t *)temp_app_struct.ctrl_data.data, ee_ctrldata_lenth );
            #endif
            WriteDataFlashWithBackup( dataflash_ctrldata_page, dataflash_ctrldata_offset, (uint8_t *)temp_app_struct.ctrl_data.data, dataflash_ctrldata_lenth );
            
            /* ...... */
            /* 注意：用户需确保在BOR复位之前执行完毕 */
            
            /* 确认SVD监测结果是否高于阈值，如否则持续等待 */
            while(false == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 20U/*us*/));
            /* 由于电源有下降到报警阈值，软件复位后重新初始化 */
            RMU_Soft_SystemReset();
        }
    }
}

int main(void)
{    
    /* 立即设置向量表 */
    SCB->VTOR = APP_START_ADDR;
    
    /* 使能IWDT */
    (void)IWDT_Init(FL_IWDT_PERIOD_8000MS);
    
    /* 延时函数初始化 */
    FL_Init();
    
    /* 使能SVD,阈值4.16V(falling)~4.284V(rising) */
    (void)SVD_Init(SVD_MONTIOR_VDD,FL_SVD_WARNING_THRESHOLD_GROUP11,FL_SVD_REFERENCE_1P0V);
    
    /* 确认SVD监测结果是否高于阈值，如否则持续等待 */
    while(false == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 2000U/*us*/));
    
    /* 使能BOR为2.0V */
    RMU_BOR_Init(FL_RMU_BOR_THRESHOLD_2P00V);
    
    bootloader_init();
    
    /* BOOT屏蔽总中断，APP此处需开启 */
    __set_PRIMASK(0);
    /* Initialize all configured peripherals */
    /* SHOULD BE KEPT!!! */
    MF_Config_Init();  /* 当使用到SVD作为唤醒源时，PMU_CR只允许配置Normal Sleep模式 */

    LED_ON(GREEN);   /* 调试信息灯 */
    FL_DelayMs(50);
    LED_OFF(GREEN);

    while(1)
    {
        /* 清狗 */
        IWDT_Clr();
        
        main_loop_timer++;
        if (main_loop_timer == VERIFY_SYSREG_TIMER)
        {
            Veri_Sys_Reg();
        }
        if (main_loop_timer == VERIFY_CTRLDATA_PARA_TIMER)
        {
            #if (EEPROM_SUPPORT == 1)
            Ctrldata_EEProm_Verify();
            #endif
            Ctrldata_DataFlash_Verify();
        }
        if (main_loop_timer >= MAIN_LOOP_TIMER_PERIOD)
        {
            main_loop_timer = 0;
        }
        
        /* 电源掉电监测处理 */
        PowerDownMonitoring();
        
        /* 功能执行 */
        if (1 == Period10msFlag) 
        {
            Period10msFlag = 0;
            /* 判断是否发生XTHF停振或PLL失锁事件，如有及时切回主系统时钟 */
            SwtichToMainSystemClock(&CMU_RCHF_Init_Data, &CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);
        }        

        /* 功能执行 */
        bootloader_process();
        
        /* 功能执行 */
        app_process();
        
        /* 功能执行 */
        #if (SLEEP_SUPPORT == 1)
        if (app_is_sleep())
        {
            sleep();
        }
        #endif
    }
}
