 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "sleep.h"
#include "lin.h"
#include "can.h"
#include "canfd.h"
#include "adc.h"
#include "dac.h"
#include "atim.h"
#include "gptim.h"
#include "i2c.h"
#include "spi.h"
#include "svd_interrupt.h"
#include "led.h"
#include "clock.h"

#if (SLEEP_SUPPORT == 1)
uint8_t  g_master_awake_init = 0;   /* 0-无sleep 1-sleep 2-awake */
uint8_t  g_slave_awake_init = 0;   /* 0-无sleep 1-sleep 2-awake */

/**
  * @brief  LIN Sleep切换
  * @param  void
  * @retval void
  */
void prepare_sleep(void)
{
    Close_None_GPIO();
    
    lin_master_sleep_init();
    lin_slave_sleep_init();
    can_sleep_init();
    canfd0_sleep_init();
    key_led_sleep_init();
    adc_sleep_init();
    dac_sleep_init();
    atim_sleep_init();
    gptim0_sleep_init();
    i2c1_sleep_init();
    spi3_sleep_init();
    
    /* 休眠模式下时钟源关闭，SVD无法实现数字滤波，禁止数字滤波 */
    FL_SVD_DisableDigitalFilter(SVD);
    /* 使能SVD唤醒功能，目的是监控电源 */
    SVD_Interrupt_Init(FL_DISABLE);

    /* 关闭监控XTHF停振检测 */
    FL_LPTIM16_Disable(LPTIM16);
    /* 清除标志 */
    FL_LPTIM16_ClearFlag_Update(LPTIM16);
    NVIC_ClearPendingIRQ(LPTIM_BSTIM_IRQn);
}

/**
  * @brief  进入 Sleep
  * @param  void
  * @retval void
  */
void enter_sleep(void)
{
    uint32_t tempWKFR;/* 唤醒标志 */
    
    g_master_awake_init = 1;
    g_slave_awake_init = 1;

    FL_RMU_PDR_Enable(RMU);  /* 使能PDR */
    FL_RMU_BOR_Enable(RMU);  /* 使能BOR */

    /* 睡眠电源模式*/
    FL_PMU_SetLowPowerMode(PMU, FL_PMU_POWER_MODE_SLEEP_OR_DEEPSLEEP);
    
    FL_IWDT_ReloadCounter(IWDT);
    __disable_irq();
    (void)__WFI();
    FL_IWDT_ReloadCounter(IWDT);
    tempWKFR = PMU->WKFR;
    (void)SelXTHFToPLL(&CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);/* 必须第一时间切换时钟 */
    FL_LPTIM16_Enable(LPTIM16);/* 使能监控XTHF停振检测 */
    
    /* 本次唤醒非唤醒源 */
    if (!(tempWKFR & PMU_WKFR_UART1WKF_Msk))
    {
        lin_master_awake_init(0);
    }
    
    if (!(tempWKFR & PMU_WKFR_UART0WKF_Msk))
    {
        lin_slave_awake_init(0);
    }

    if (!(tempWKFR & PMU_WKFR_FSCANWKF_Msk))
    {
        can_awake_init(0);
    }

    if (!(tempWKFR & PMU_WKFR_CANFD0WKF_Msk))
    {
        canfd0_awake_init(0);
    }

    if (!(tempWKFR & PMU_WKFR_IOWKF_Msk))
    {
        key_led_awake_init(0);
    }     
    __enable_irq();
       
    /* 非唤醒源 */
    adc_awake_init(0);
    dac_awake_init(0);
    atim_awake_init(0);
    gptim0_awake_init(0);
    i2c1_awake_init(0);
    spi3_awake_init(0);
    
    if (SVD_HIGHER_THRESHOLD == SVD_IT())
    {
        /* LIN 唤醒 */
        LED_ON(BLUE);   /* 调试信息灯 */
        FL_DelayMs(50);
        LED_OFF(BLUE); 
    }
    else
    {
        /* SVD监测电源下降中断唤醒， 及时掉电处理 */
        /* ...... */
        LED_ON(RED);  /* 调试信息灯 */
    }
    
    /* SVD禁止中断 */
    SVD_Interrupt_DeInit();
    /* SVD恢复初始配置 */
    SVD_Init(SVD_MONTIOR_VDD, FL_SVD_WARNING_THRESHOLD_GROUP11, FL_SVD_REFERENCE_1P0V);
}

uint8_t app_is_sleep(void)
{
    if (lin_master_is_sleep() &&
        lin_slave_is_sleep() &&
        can_is_sleep() &&
        canfd0_is_sleep() &&
        key_led_is_sleep() &&
        adc_is_sleep() &&
        dac_is_sleep() &&
        atim_is_sleep() &&
        gptim0_is_sleep() &&
        i2c1_is_sleep() &&
        spi3_is_sleep())
    { 
        return 1;
    }
    else
    {
        return 0;
    }
}

/**
  * @brief  lin Sleep
  * @param  void
  * @retval void
  */
void sleep(void)
{
    prepare_sleep();
    enter_sleep();
}
#endif  /* End #if (SLEEP_SUPPORT == 1) */
